import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import mgui.io.util.IoFunctions;
import mgui.numbers.MguiDouble;
import mgui.numbers.MguiInteger;
import mgui.util.StringFunctions;
import Jama.Matrix;


public class Temp10 {

	public static void main(String[] args) throws Exception{
		
		File fsl_dir = new File("C:\\Users\\Andrew\\Documents\\Projects\\ICBM\\85\\fsl\\probtrackx");
		String subject_total_pattern = "{fsl_dir}\\{subject}\\connectivity_total.txt";
		String subject_mean_pattern = "{fsl_dir}\\{subject}\\connectivity_mean.txt";
		File total_file_all = new File(fsl_dir + "\\connectivity_total_all.txt");
		File mean_file_all = new File(fsl_dir + "\\connectivity_mean_all.txt");
		
		String matrix_pattern = "{fsl_dir}\\{subject}\\{roi}\\matrix_seeds_to_all_targets";
		
		File name_file = new File(fsl_dir + "\\rm_names.txt");
//		NameMapLoader loader = new NameMapLoader(name_map_file);
//		NameMap name_map = (NameMap)loader.loadObject();
//		ArrayList<String> names = new ArrayList<String>(name_map.getNames());
//		Collections.sort(names);
		
//		BufferedWriter writer1 = new BufferedWriter(new FileWriter(new File(fsl_dir + "\\rm_names.txt")));
//		for (int i = 0; i < names.size(); i++)
//			writer1.write(names.get(i) + "\n");
//		
//		writer1.close();
		ArrayList<String> names = new ArrayList<String>();
		BufferedReader reader = new BufferedReader(new FileReader(name_file));
		String line = reader.readLine();
		while (line != null){
			names.add(line);
			line = reader.readLine();
			}
		
		String[] subjects = fsl_dir.list(IoFunctions.getDirFilter());
		
		ArrayList<Matrix> subject_matrix_total = new ArrayList<Matrix>();
		ArrayList<Matrix> subject_matrix_mean = new ArrayList<Matrix>();
		
		// For each subject, compile a connectivity matrix
		for (int i = 0; i < subjects.length; i++){
			
			String subject = subjects[i];
			System.out.print("Reading subject '" + subject + "'...");
			Matrix s_matrix = new Matrix(names.size(), names.size());
			Matrix n_matrix = new Matrix(names.size(), names.size());
			subject_matrix_total.add(s_matrix);
			subject_matrix_mean.add(n_matrix);
			
			// For each ROI, update connectivity matrix
			for (int j = 0; j < names.size(); j++){
				String roi_j = names.get(j);
				
				String fname = StringFunctions.replace(matrix_pattern, "{fsl_dir}", fsl_dir.getAbsolutePath());
				fname = StringFunctions.replace(fname, "{subject}", subject);
				fname = StringFunctions.replace(fname, "{roi}", roi_j);
				MguiInteger tract_count = new MguiInteger();
				ArrayList<Integer> totals = getSeedToTargetTotals(new File(fname), names.size() - 1, tract_count);
				
				int _k;
				for (int k = 0; k < names.size(); k++){
					_k = k;
					if (k > j) _k--;
					if (k != j){
						if (tract_count.getValue() == 0){
							s_matrix.set(j, k, 0);
							n_matrix.set(j, k, 0);
						}else{
							s_matrix.set(j, k, totals.get(_k));
							n_matrix.set(j, k, (double)totals.get(_k) / tract_count.getValue());
							}
						}
					}
				}
			
			System.out.println("done.");
			
			}
		
		// Done reading; now write results
		Matrix s_matrix_all = new Matrix(names.size(), names.size());
		Matrix n_matrix_all = new Matrix(names.size(), names.size());
		
		for (int i = 0; i < subjects.length; i++){
			
			String subject = subjects[i];
			System.out.print("Writing subject '" + subject + "'...");
			
			String s_file = StringFunctions.replace(subject_total_pattern, "{fsl_dir}", fsl_dir.getAbsolutePath());
			s_file = StringFunctions.replace(s_file, "{subject}", subject);
			Matrix s_matrix = subject_matrix_total.get(i);
			s_matrix_all.plusEquals(s_matrix);
			writeMatrix(s_matrix, new File(s_file), "0");
			
			String n_file = StringFunctions.replace(subject_mean_pattern, "{fsl_dir}", fsl_dir.getAbsolutePath());
			n_file = StringFunctions.replace(n_file, "{subject}", subject);
			Matrix n_matrix = subject_matrix_mean.get(i);
			n_matrix_all.plusEquals(n_matrix);
			writeMatrix(n_matrix, new File(n_file), "0.0000");
			
			System.out.println("done.");
			}
		
		n_matrix_all.times(1.0 / (double)subjects.length);
		
		writeMatrix(s_matrix_all, total_file_all, "0");
		writeMatrix(n_matrix_all, mean_file_all, "0.0000");
		
		System.out.println("\nAll done.");
		
	}
	
	static ArrayList<Integer> getSeedToTargetTotals(File file, int n, MguiInteger count) throws IOException{
		
		BufferedReader reader = new BufferedReader(new FileReader(file));
		String line = reader.readLine();
		
		ArrayList<Integer> totals = new ArrayList<Integer>(n);
		for (int i = 0; i < n; i++)
			totals.add(0);
		
		int i = 0;
		while (line != null){
			String[] parts = line.split("  ");
			if (parts.length != n){
				reader.close();
				throw new IOException("Invalid # tokens in line " + i + " of '" + file.getAbsolutePath() + "'.");
				}
			
			for (int j = 0; j < n; j++){
				totals.set(j, totals.get(j) + Integer.valueOf(parts[j]));
				}
			
			line = reader.readLine();
			i++;
			}
		
		reader.close();
		
		count.setValue(i);
		return totals;
		
	}
	
	static void writeMatrix(Matrix matrix, File file, String format) throws IOException{
		
		BufferedWriter writer = new BufferedWriter(new FileWriter(file));
		int n = matrix.getRowDimension();
		int m = matrix.getColumnDimension();
		
		for (int i = 0; i < n; i++){
			for (int j = 0; j < m; j++){
				if (j != 0)
					writer.write(" ");
				writer.write(MguiDouble.getString(matrix.get(i, j), format));
				}
			writer.write("\n");
			}
		writer.close();
	}
	
	
}
