package minc;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.TreeSet;

import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import mgui.command.CommandFunctions;
import mgui.command.CommandFunctions.ProcessState;
import mgui.command.CommandInstance;
import mgui.geometry.Box3D;
import mgui.geometry.Grid3D;
import mgui.geometry.Mesh3D;
import mgui.geometry.Mesh3D.MeshFace3D;
import mgui.geometry.mesh.MeshFunctions;
import mgui.geometry.volume.VolumeFunctions;
import mgui.interfaces.logs.LoggingType;
import mgui.interfaces.maps.NameMap;
import mgui.interfaces.shapes.Mesh3DInt;
import mgui.interfaces.shapes.ShapeSet3DInt;
import mgui.interfaces.shapes.Volume3DInt;
import mgui.interfaces.shapes.volume.VolumeMetadata;
import mgui.io.domestic.maps.NameMapLoader;
import mgui.io.domestic.shapes.ShapeIOException;
import mgui.io.domestic.shapes.SurfaceInputOptions;
import mgui.io.domestic.shapes.VolumeInputOptions;
import mgui.io.domestic.shapes.VolumeOutputOptions;
import mgui.io.foreign.minc.MincSurfaceDataLoader;
import mgui.io.foreign.minc.MincSurfaceDataWriter;
import mgui.io.foreign.minc.MincSurfaceLoader;
import mgui.io.standard.nifti.Nifti1Dataset;
import mgui.io.standard.nifti.NiftiMetadata;
import mgui.io.standard.nifti.NiftiVolumeLoader;
import mgui.io.standard.nifti.NiftiVolumeWriter;
import mgui.io.util.IoFunctions;
import mgui.numbers.MguiDouble;
import mgui.numbers.MguiFloat;
import mgui.numbers.MguiInteger;
import mgui.numbers.MguiNumber;

/**************************************************************
 * Functions for performing Minc/Civet functions on a dataset.
 * 
 * @author Andrew Reid
 *
 */
public class MincFunctions extends CommandInstance {

	public static String path = null;
	
	public static void main(String[] args) throws Exception{
		
		MincFunctions instance = new MincFunctions();
		instance.execute(args);
		
	}
	
	@Override
	protected boolean run_command(String command) {
		
		if (command.equals("set_path")){
			// Set the path to use in command calls. No value unsets the path.
			path = parameters.get("path");
			return true;
			}
		
		if (command.equals("adni_nii2mnc")){
			return adni_nii2mnc();
			}
		
		if (command.equals("minc_operation_subjects")){
			return minc_operation_subjects();
			}
		
		if (command.equals("minc_operation_images")){
			return minc_operation_images();
			}
		
		if (command.equals("adni_extract_xml_values")){
			return adni_extract_xml_values();
			}
		
		if (command.equals("adni_create_symlinks")){
			return adni_create_symlinks();
			}
		
		if (command.equals("adni_create_input_dirs")){
			return adni_create_input_dirs();
			}
		
		if (command.equals("adni_move_valid_subjects")){
			return adni_move_valid_subjects();
			}
		
		if (command.equals("adni_strip_cbrain_names")){
			return adni_strip_cbrain_names();
			}
		
		if (command.equals("adni_move_valid_images")){
			return adni_move_valid_images();
			}
		
		if (command.equals("adni_validation_report")){
			return adni_validation_report();
			}
		
		if (command.equals("civet_failures")){
			return civet_failures();
			}
		
		if (command.equals("civet_move_no_logs")){
			return civet_move_no_logs();
			}
		
		if (command.equals("adni_compute_scale_transforms")){
			return adni_compute_scale_transforms();
			}
		
		if (command.equals("adni_transform_images")){
			return adni_transform_images();
			}
		
		if (command.equals("surface_intersections_report")){
			return surface_intersections_report();
			}
		
		if (command.equals("surface_intersections_report_from_files")){
			return surface_intersections_report_from_files();
			}
		
		if (command.equals("civet_average_image")){
			return civet_average_image();
			}
		
		if (command.equals("civet_surface_atlas_to_volume")){
			return civet_surface_atlas_to_volume();
			}
		
		if (command.equals("create_volume_atlas_masks")){
			return create_volume_atlas_masks();
			}
		
		log("MincFunctions: no such command '" + command + "'.",
				LoggingType.Errors);
		return false;
	}
		
	/***************************************
	 * Constructs an instance with the given arguments.
	 * 
	 * @param args
	 */
	public MincFunctions(){
		
		
		
	}
	
	/*************************************************
	 * Runs a conversion from Nifti to Minc format, using the nii2mnc command from the Minc library.
	 * This command is specific for the ADNI dataset; it will search the directory specified by the
	 * <code>source_dir</code> parameter, compile a list of subjects, and create a single subject directory
	 * for each, within the directory specified by the <code>target_dir</code> parameter. This directory
	 * will be created if it doesn't already exist.
	 * 
	 * <p>Within each subject directory, the subdirectory for a given preprocessing state (specified by
	 * the <code>process_id</code> parameter will be searched, and a single mnc file will be created for 
	 * each ADNI visit. The files will be named as: 
	 * 
	 * <p>[<code>prefix</code>]_[subject_id]_[acquisition_id]_[<code>suffix</code>].mnc. 
	 * 
	 * <p>Note the processing id is not included in this name; appropriately named target directories should be used
	 * if necessary. 
	 * 
	 * <p>The arguments for this function are (params with dashes must include name):
	 * 
	 * <ul>
	 * <li><code>process_id</code> - Identifier for processing step; use "" for none, in which case all directories are
	 * processed.
	 * <li><code>source_dir</code> - The source directory
	 * <li><code>target_dir</code> - The target directory
	 * <li><code>prefix</code> - The minc file prefix
	 * <li><code>suffix</code> The minc file suffix
	 * <li><code>-param_file</code> [optional] - A delimited text file specifying the parameters to pass to 
	 * <code>nii2mnc</code>.
	 * <li><code>-skip_existing</code> [optional] - Skips any subjects existing in the target directory (false
	 * by default). 
	 * </ul>
	 * 
	 * @param args
	 */
	boolean adni_nii2mnc(){
		
		log("Starting adni_nii2mnc.");
		
		boolean skip_existing = parameters.containsKey("skip_existing");
		
		//Get processing state id
		String process_id = parameters.get("process_id"); 
		if (process_id != null && !process_id.endsWith(File.separator))
			process_id = process_id + File.separator;
		
		//Does source dir exist?
		String _param = parameters.get("source_dir");
		if (_param == null){
			System.out.println("adni_nii2mnc: source_dir is a required parameter.");
			return false;
			}
			
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			System.out.println("adni_nii2mnc: Cannot find source dir: " + source_dir.getAbsolutePath());
			return false;
			}
		
		//Create target dir if it doesn't exist
		_param = parameters.get("target_dir");
		if (_param == null){
			System.out.println("adni_nii2mnc: target_dir is a required parameter.");
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists()){
			if (!target_dir.mkdir()){
				System.out.println("adni_nii2mnc: Cannot create target dir: " + target_dir.getAbsolutePath());
				return false;
				}
		}else{
			if (!target_dir.isDirectory()){
				System.out.println("adni_nii2mnc: Target dir is not a directory: " + target_dir.getAbsolutePath());
				return false;
				}
			}
		
		//Does param file exist if specified?
		String[] params = new String[]{"nii2mnc"};
		_param = parameters.get("param_file");
		if (_param != null){
			File param_file = new File(_param);
			if (!param_file.exists()){
				System.out.println("adni_nii2mnc: Cannot find parameter file: " + param_file.getAbsolutePath());
				return false;
				}
			try{
				String[] params2 = getParametersFromFile(param_file);
				params = new String[params2.length + 1];
				params[0] = "nii2mnc";
				System.arraycopy(params2, 0, params, 1, params2.length);
			}catch (IOException ex){
				ex.printStackTrace();
				return false;
				}
			}
		
		String prefix = parameters.get("prefix");
		if (prefix == null) prefix = "";
		String suffix = parameters.get("suffix");
		if (suffix == null) suffix = "";
		
		//Get list of subjects in source dir
		String[] subjects = source_dir.list(IoFunctions.getDirFilter());
		
		if (subjects.length == 0){
			System.out.println("adni_nii2mnc: No subjects found in source dir: " + source_dir.getAbsolutePath());
			return false;
			}
		
		int success_count = 0, fail_count = 0, skipped_count = 0;
		
		//MincFunctions system = new MincFunctions();
		
		//For each subject, within specified processing directory
		for (int i = 0; i < subjects.length; i++){
			String subject_id = subjects[i];
			String[] subs = null;
			if (process_id == null){
				File subject_dir = new File(source_dir + File.separator + subject_id);
				subs = subject_dir.list(IoFunctions.getDirFilter());
			}else{
				subs = new String[]{process_id};	
				}
			
			File target_subject = new File(target_dir.getAbsolutePath() + File.separator + subject_id);
			boolean _continue = true, skipped = false;
			if (!target_subject.exists()){
				if (!target_subject.mkdir()){
					System.out.println("adni_nii2mnc: Cannot create subject dir '" + target_subject.getAbsolutePath() + "'.");
					_continue = false;
					}
			} 
			
			boolean success = true;
			
			//1. For each processing state
			for (int p = 0; _continue && p < subs.length; p++){
				File process_dir = new File(source_dir + File.separator + subject_id + File.separator + subs[p]); 
				//1.1. For each timestamp (visit)
				String[] visits = process_dir.list(IoFunctions.getDirFilter());
				if (visits == null){
					success = false;
					}
				for (int t = 0; success && t < visits.length; t++){
					File visit_dir = new File(process_dir.getAbsolutePath() + File.separator + visits[t]);
					String[] acquisitions = visit_dir.list(IoFunctions.getDirFilter());
					//1.1.1. For each acquisition ID
					if (acquisitions == null){
						success = false;
						}
					for (int a = 0; success && a < acquisitions.length; a++){
						//1.1.1.1. Convert Nifti images -> target directory
						File a_dir = new File(visit_dir.getAbsolutePath() + File.separator + acquisitions[a]);
						String[] images = a_dir.list(getNiftiFilter());
						
						if (images == null){
							success = false;
							}
						
						for (int j = 0; success && j < images.length; j++){
							String image_out = target_subject.getAbsolutePath() + File.separator + 
											   prefix + "_" + subject_id + "_" + 
											   acquisitions[a] + "_" + suffix + ".mnc";
							File f = new File(image_out);
							if (!f.exists() || !skip_existing){
								String image_in = a_dir.getAbsolutePath() + File.separator + images[j];
								String[] _params = new String[params.length + 2];
								System.arraycopy(params, 0, _params, 0, params.length);
								_params[params.length] = image_in;
								_params[params.length + 1] = image_out;
								
								//long elapsed = execute(_params);
								String command = getCommandLine(_params);
								ProcessState state = ProcessState.NotStarted;
								
								try{
									state = CommandFunctions.callAndWait(command);
//									CommandFunctions.call(command);
								}catch (IOException ex){
									state = ProcessState.TerminatedWithError;
									success = false;
									}
								
								if (state == ProcessState.TerminatedNormally){
									log("adni_nii2mnc: Converted '" + image_out + ".");
									//elapsed_total += elapsed;
									//success = true;
									}
							}else{
								log("adni_nii2mnc: Skipping existing file '" + image_out + "'.");
								skipped = true;
								skipped_count++;
								}
							}
						}
					}
				}
			/*if (skipped){
				skipped_count++;
				log("adni_nii2mnc: Skipped existing subject '" + subject_id + "'.");
			}else */ if (success){
				success_count++;
			}else{
				fail_count++;
				log("Subject '" + subject_id + " failed.");
				}
			}
		
		log("adni_nii2mnc: Done. " + success_count + " successes. " + fail_count + " failures. " +
			skipped_count + " skipped.");
		
		return fail_count == 0;
		
	}
	
	/*********************************************
	 * Runs a minc operation on a source directory, organized by subject_id, with source images at a depth of one.
	 * If the target directory is the same as the source directory, source images will be overwritten with the result.
	 * Operation must accept parameters in the form [command] [options] [source_image] [target_image].
	 * 
	 * <p>Parameters are:
	 * 
	 * <ol>
	 * <li><code>operation</code> - Minc operation to be performed.
	 * <li><code>source_dir</code> - Directory containing the source images, with a single subdirectory for each subject.
	 * <li><code>target_dir</code> [Optional] - Directory to store the resulting images, with a single subdirectory for each subject.
	 * <li><code>source_ext</code> [Optional] - Extension for source image (default is "mnc")
	 * <li><code>target_ext</code> [Optional] - Extension for target image (default is "mnc")
	 * <li><code>param_file</code> - [Optional] - Delimited text file containing the parameters for the command.
	 * <li><code>source_subdir</code> [optional] - Subdirectory within source subject directory, containing the image files.
	 * <li><code>target_subdir</code> [optional] - Subdirectory within target subject directory, containing the image files.
	 * <li><code>pattern</code> [optional] - Pattern to match for the source image files. If not specified, processes all files.
	 * <li>skip_existing 		[optional] - Indicates that existing target files should be skipped
	 * </ol>
	 * 
	 * @param args
	 */
	boolean minc_operation_subjects(){
		
		log("Starting minc_operation.");
		
		boolean skip_existing = parameters.containsKey("skip_existing");
		
		String operation = parameters.get("operation");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("minc_operation_subjects: source_dir is a required parameter.", LoggingType.Errors);
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("minc_operation_subjects: Cannot find source dir: " + source_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		//Create target dir if it doesn't exist
		boolean has_target = parameters.containsKey("target_dir");
		
		File target_dir = null; 
		
		if (has_target){
			_param = parameters.get("target_dir");
			target_dir = new File(_param);
			if (!target_dir.exists()){
				if (!target_dir.mkdir()){
					log("minc_operation_subjects: Cannot create target dir: " + target_dir.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
			}else{
				if (!target_dir.isDirectory()){
					log("minc_operation_subjects: Target dir is not a directory: " + target_dir.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
				}
			}
		
		String[] params = new String[]{operation};
		_param = parameters.get("param_file");
		
		if (_param != null){
			File param_file = new File(_param);
			if (!param_file.exists()){
				log("minc_operation_subjects: Cannot find parameter file: " + param_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			try{
				String[] params2 = getParametersFromFile(param_file);
				params = new String[params2.length + 1];
				params[0] = operation;
				System.arraycopy(params2, 0, params, 1, params2.length);
			}catch (IOException ex){
				log("minc_operation_subjects: Error loading parameter file:\nDetails: " + ex.getMessage(),
						LoggingType.Errors);
				//ex.printStackTrace();
				return false;
				}
			}
		
		String source_subdir = parameters.get("source_subdir");
		if (source_subdir != null && !source_subdir.startsWith(File.separator))
			source_subdir = File.separator + source_subdir;
		if (source_subdir == null) source_subdir = "";
		
		String target_subdir = parameters.get("target_subdir");
		if (target_subdir != null && !target_subdir.startsWith(File.separator))
			target_subdir = File.separator + target_subdir;
		if (target_subdir == null) target_subdir = "";
		
		//Get list of subjects in source dir
		String[] subjects = source_dir.list(IoFunctions.getDirFilter());
		
		if (subjects.length == 0){
			log("minc_operation_subjects: No subjects found in source dir: " + source_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		String pattern = parameters.get("pattern");
		String source_ext = "mnc";
		_param = parameters.get("source_ext");
		if (_param != null)
			source_ext = _param;
		String target_ext = "mnc";
		_param = parameters.get("target_ext");
		if (_param != null)
			target_ext = _param;
		
		int success_count = 0, fail_count = 0, skip_count = 0;
		
		//For each subject, within specified processing directory
		for (int i = 0; i < subjects.length; i++){
			
			if (this.is_interrupted){
				log("minc_operation_subjects: Process was interrupted.",
						LoggingType.Errors);
				return false;
				}
			
			boolean skip = false;
			
			String subject_id = subjects[i];
			log("minc_operation_subjects: Processing subject '" + subject_id + "'.");
			File current_dir = new File(source_dir.getAbsolutePath() + File.separator + subject_id + source_subdir);
			File write_dir = null;
			
			boolean _continue = true;
			if (target_dir != null){
				write_dir = new File(target_dir.getAbsolutePath() + File.separator + subject_id);
				if (!write_dir.exists() && !write_dir.mkdir()){
					_continue = false;
				}else if (target_subdir.length() > 0){
					write_dir = new File(target_dir.getAbsolutePath() + File.separator + subject_id + target_subdir);
					if (!write_dir.exists() && !write_dir.mkdir())
						_continue = false;
					}
				}
			
			boolean success = false;
			String[] images = current_dir.list(getExtFilter(source_ext, pattern));
			
			for (int j = 0; j < images.length && _continue; j++){
				
				if (this.is_interrupted){
					log("minc_operation_subjects: Process was interrupted.",
							LoggingType.Errors);
					return false;
					}
				
				int n = params.length;
				int m = n + 2;
				if (!has_target) m = n + 1; 
				String[] _params = new String[m];
				System.arraycopy(params, 0, _params, 0, params.length);
				_params[n] = current_dir.getAbsolutePath() + File.separator + images[j];
				if (has_target){
					if (_params[n].equals(_params[n + 1])){
						//doesn't like the same image so, use temp here
						_params[n + 1] = write_dir.getAbsolutePath() + File.separator + "temp." + target_ext;
					}else{
						String img = images[j];
						if (!source_ext.equals(target_ext))
							img = img.replace(source_ext, target_ext);
						_params[n + 1] = write_dir.getAbsolutePath() + File.separator + img;
						File file = new File(_params[n + 1]);
						if (file.exists()){
							if (!skip_existing){
								file.delete();
							}else{
								log("minc_operation_subjects: Target file exists; skipping.");
								skip = true;
								}
							}
						}
					}
				
				if (!skip){
					String command = getCommandLine(_params);
					ProcessState state = ProcessState.NotStarted;
					
					try{
						state = CommandFunctions.callAndWait(command);
					}catch (IOException ex){
						state = ProcessState.TerminatedWithError;
						}
					
					if (state == ProcessState.TerminatedNormally){
						try{
							if (has_target && _params[n + 1].endsWith("temp." + target_ext)){
								//copy temp file
								File temp = new File(_params[n + 1]);
								IoFunctions.copyFile(temp, new File(_params[n]));
								temp.delete();
								}
						
							log("minc_operation_subjects: Processed '" + _params[n] + "'.");
							success = true;
						}catch (IOException ex){
							log("minc_operation_subjects: Problem converting temp file '" + _params[n + 1] + "'.",
									LoggingType.Errors);
							}
						}
					}
				}
			if (success){
				success_count++;
			}else if (skip){
				skip_count++;
			}else{
				fail_count++;
				log("Subject '" + subject_id + " failed.", LoggingType.Errors);
				}
			}
		
		log("minc_operation_subjects: Done. " + success_count + " successes. " + fail_count + " failures. " + skip_count + " skipped.");
		
		return fail_count == 0;
	}
	
	@Override
	protected String getCommandLine(String[] args){
		String command = super.getCommandLine(args);
		command = command.trim();
		if (path == null) return command;
		path = path.trim();
		if (!path.endsWith(File.separator))
			path = path + File.separator;
		
		command = path + command;
		return command;
	}
	
	/*********************************************
	 * Runs a minc operation on a directory containing source images.
	 * If the target directory is the same as the source directory, source images will be overwritten with the result.
	 * Operation must accept parameters in the form [command] [options] [source_image] [target_image].
	 * 
	 * <p>Parameters are:
	 * 
	 * <ol>
	 * <li><code>operation</code> - Minc operation to be performed.
	 * <li><code>source_dir</code> - Directory containing the source images.
	 * <li><code>target_dir</code> [optional] - Directory to store the resulting images.
	 * <li><code>param_file</code> - [Optional] Delimited text file containing the parameters for the command.
	 * </ol>
	 * 
	 * @param args
	 */
	boolean minc_operation_images(){
		
		log("Starting minc_operation_images.");
		
		String operation = parameters.get("operation");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("minc_operation_images: source_dir is a required parameter.");
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("minc_operation_images: Cannot find source dir: " + source_dir.getAbsolutePath());
			return false;
			}
		
		//Create target dir if it doesn't exist
		boolean has_target = parameters.containsKey("target_dir");
		
		File target_dir = null; 
		
		if (has_target){
			_param = parameters.get("target_dir");
			target_dir = new File(_param);
			if (!target_dir.exists()){
				if (!target_dir.mkdir()){
					log("minc_operation_images: Cannot create target dir: " + target_dir.getAbsolutePath());
					return false;
					}
			}else{
				if (!target_dir.isDirectory()){
					log("minc_operation_images: Target dir is not a directory: " + target_dir.getAbsolutePath());
					return false;
					}
				}
			}
		
		String[] params = new String[]{operation};
		_param = parameters.get("param_file");
		
		if (_param != null){
			File param_file = new File(_param);
			if (!param_file.exists()){
				log("minc_operation_images: Cannot find parameter file: " + param_file.getAbsolutePath());
				return false;
				}
			try{
				String[] params2 = getParametersFromFile(param_file);
				params = new String[params2.length + 1];
				params[0] = operation;
				System.arraycopy(params2, 0, params, 1, params2.length);
			}catch (IOException ex){
				ex.printStackTrace();
				return false;
				}
			}
		
		//Get list of subjects in source dir
		String[] images = source_dir.list(getMincFilter());
		
		if (images.length == 0){
			log("minc_operation_images: No images found in source dir: " + source_dir.getAbsolutePath());
			return false;
			}
		
		int success_count = 0, fail_count = 0;
		
		//For each subject, within specified processing directory
		for (int i = 0; i < images.length; i++){
			String image = images[i];
			//File current_dir = new File(source_dir.getAbsolutePath() + File.separator + subject_id + subdir);
			
			boolean success = false;
			
			int n = params.length;
			int m = n + 2;
			if (!has_target) m = n + 1; 
			String[] _params = new String[m];
			System.arraycopy(params, 0, _params, 0, params.length);
			_params[n] = source_dir.getAbsolutePath() + File.separator + image;
			if (has_target)
				_params[n + 1] = target_dir.getAbsolutePath() + File.separator + image;
			
			//doesn't like the same image so, use temp here
			if (has_target && _params[n].equals(_params[n + 1])){
				_params[n + 1] = target_dir.getAbsolutePath() + File.separator + "temp.mnc";
				}
			
			String command = getCommandLine(_params);
			ProcessState state = ProcessState.NotStarted;
			
			try{
				state = CommandFunctions.callAndWait(command);
//				CommandFunctions.call(command);
			}catch (IOException ex){
				state = ProcessState.TerminatedWithError;
				}
			
			if (state == ProcessState.TerminatedNormally){
				try{
					if (has_target && _params[n + 1].endsWith("temp.mnc")){
						//copy temp file
						File temp = new File(_params[n + 1]);
						IoFunctions.copyFile(temp, new File(_params[n]));
						temp.delete();
						}
				
					log("minc_operation_images: Processed '" + _params[n] + "'.");
					success = true;
				}catch (IOException ex){
					log("minc_operation_images: Problem converting temp file '" + _params[n + 1] + "'.");
					}
				}
			if (success){
				success_count++;
			}else{
				fail_count++;
				log("File '" + image + "' failed.");
				}
			}
		
		log("minc_operation_images: Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
		
	}
	
	/****************************************************************
	 * Creates a set of symbolic links to ADNI volumes, which can be stored in a single directly and
	 * passed to the Civet pipeline as individual instances. Creates a single symlink for each volume
	 * in the subdirectories of <code>source_dir</code>.
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li><code>source_dir</code> - The directory containing the ADNI images.
	 * <li><code>target_dir</code> - The directory in which to write the symlinks.
	 * </ul>
	 * 
	 */
	boolean adni_create_symlinks(){
		
		log("Starting adni_create_symlinks.");
				
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("adni_create_symlinks: source_dir is a required parameter.");
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("adni_create_symlinks: Cannot find source dir: " + source_dir.getAbsolutePath());
			return false;
			}
		
		_param = parameters.get("target_dir");
		if (_param == null){
			log("adni_create_symlinks: target_dir is a required parameter.");
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists() && !target_dir.mkdir()){
			log("adni_create_symlinks: Cannot create target dir: " + target_dir.getAbsolutePath());
			return false;
			}
		
		String[] subjects = source_dir.list(IoFunctions.getDirFilter());
		int success_count = 0, fail_count = 0;
		
		for (int s = 0; s < subjects.length; s++){
			String subject_id = subjects[s];
			
			File subject_dir = new File(source_dir.getAbsolutePath() + File.separator + subject_id);
			String[] images = subject_dir.list(getMincFilter());
			
			for (int i = 0; i < images.length; i++){
				String image = images[i];
				
				String command = "ln -s " + subject_dir.getAbsolutePath() + File.separator + image + " " +
								 target_dir.getAbsolutePath() + File.separator + image;
				
				ProcessState state = ProcessState.NotStarted;
				
				try{
					state = CommandFunctions.callAndWait(command);
//					CommandFunctions.call(command);
				}catch (IOException ex){
					state = ProcessState.TerminatedWithError;
					}
				
				if (state == ProcessState.TerminatedNormally){
					log("adni_create_symlinks: Created link for '" + image + "'.");
					success_count++;
				}else{
					log("adni_create_symlinks: Error creating link for '" + image + "'.");
					fail_count++;
					}
				}
			}
		
		log("adni_create_symlinks: Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
	}
	
	/*****************************
	 * Loads parameters from a text file; each line holds a space-delimited parameter of the form:
	 * 
	 * <p>Value: [parameter_name] [value_1] [value_2] ... [value_n]
	 * <p>Flag: [parameter_name]
	 * 
	 * @param parameter_file
	 * @return
	 */
	static String[] getParametersFromFile(File parameter_file) throws IOException{
		
		BufferedReader reader = new BufferedReader(new FileReader(parameter_file));
		
		ArrayList<String> params = new ArrayList<String>();
		String line = reader.readLine();
		
		while (line != null){
			if (!line.contains(" ")){
				params.add("-" + line);
			}else{
				String p = line.substring(0, line.indexOf(" "));
				params.add("-" + p);
				params.add(line.substring(line.indexOf(" ") + 1));
				}
			line = reader.readLine();
			}
		
		String[] array = new String[params.size()];
		for (int i = 0; i < params.size(); i++){
			array[i] = params.get(i);
			}
		
		return array;
	}
	
	/***************************************************************************
	 * Searches the XML data in an ADNI directory to extract values from XML files. Stores the
	 * results in a tab-delimited text file. This function does a naive search for the first instance
	 * of a specific text pattern in the file, and returns the wrapped value on that line. It is
	 * not a robust XML parser.
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li><code>source_dir</code> - Directory containing the XML files
	 * <li><code>output_file</code> - File to store the extracted field strengths
	 * <li><code>values_file</code> - File containing search patterns identifying the values to be extracted (one
	 * per line). Each tab-delimited line must be of the format: [value name][tab=\t][search pattern]
	 * </ul>
	 * 
	 */
	boolean adni_extract_xml_values(){
		
		log("Starting adni_extract_xml_values.");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			System.out.println("adni_extract_xml_values: source_dir is a required parameter.");
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("adni_extract_xml_values: Cannot find source dir: " + source_dir.getAbsolutePath(), LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("output_file");
		if (_param == null){
			log("adni_extract_xml_values: output_file is a required parameter.", LoggingType.Errors);
			return false;
			}
		
		File output_file = new File(_param);
		if (output_file.exists() && !output_file.delete()){
			log("adni_extract_xml_values: Could not overwrite output file '" + output_file.getAbsolutePath() +"'.", LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("values_file");
		if (_param == null){
			log("adni_extract_xml_values: values_file is a required parameter.", LoggingType.Errors);
			return false;
			}
		
		File values_file = new File(_param);
		if (!values_file.exists()){
			log("adni_extract_xml_values: Could not find values file '" + values_file.getAbsolutePath() +"'.", LoggingType.Errors);
			return false;
			}
		
		ArrayList<String> values = new ArrayList<String>();
		ArrayList<String> patterns = new ArrayList<String>();
		
		int current_line = 0;
		
		//Get patterns from values file
		try{
			BufferedReader reader = new BufferedReader(new FileReader(values_file));
			String line = reader.readLine();
			current_line++;
			
			while (line != null){
				StringTokenizer tokens = new StringTokenizer(line, "\t");
				values.add(tokens.nextToken());
				patterns.add(tokens.nextToken());
				line = reader.readLine();
				current_line++;
				}
			
			reader.close();
		}catch (IOException ex){
			log("adni_extract_xml_values: Could not read values file '" + values_file.getAbsolutePath() +"'.", LoggingType.Errors);
			return false;
		}catch(NoSuchElementException ex){
			log("adni_extract_xml_values: Error reading values file '" + values_file.getAbsolutePath() +"'; line " + current_line + ".", 
				LoggingType.Errors);
			return false;
			}
		
		if (values.size() == 0){
			log("adni_extract_xml_values: No values specified in values file.", LoggingType.Errors);
			return false;
			}
		
		//Do it.
		int success_count = 0, fail_count = 0;
		BufferedWriter writer = null;
		
		try{
			writer = new BufferedWriter(new FileWriter(output_file)); 
			writer.write("Subject");
			for (int v = 0; v < values.size(); v++){
				writer.write("\t" + values.get(v));
				}
		}catch (IOException ex){
			log("adni_extract_xml_values: Could not write to output file '" + output_file.getAbsolutePath() +"'.", LoggingType.Errors);
			}
		
		String[] files = source_dir.list(getXmlFilter());
		for (int i = 0; i < files.length; i++){
			File input = new File(source_dir.getAbsolutePath() + File.separator + files[i]);
			
			try{
				BufferedReader reader = new BufferedReader(new FileReader(input)); 
				String line = reader.readLine();
				String subject = null, series = null;
				HashMap<String, String> value_map = new HashMap<String, String>();
				boolean done = false;
				
				while (!done){
					if (line.contains("<subjectIdentifier>")){
						String s = line.substring(line.indexOf(">") + 1);
						subject = s.substring(0, s.indexOf("<"));
						}
					
					if (line.contains("<seriesIdentifier>")){
						String s = line.substring(line.indexOf(">") + 1);
						series = s.substring(0, s.indexOf("<"));
						}
					
					for (int v = 0; v < values.size(); v++){
						String value = values.get(v);
						String pattern = patterns.get(v);
						if (line.contains(pattern)){
							String s = line.substring(line.indexOf(">") + 1);
							s = s.substring(0, s.indexOf("<"));
							value_map.put(value, s);
							}
						}
					
					line = reader.readLine();
					if (line == null){
						done = true;
					}else if (subject != null){
						if (value_map.size() == values.size())
							done = true;
						}
					}
				
				if (subject != null){
					writer.write("\n" + subject + "_S" + series);
					for (int v = 0; v < values.size(); v++){
						String s = value_map.get(values.get(v));
						if (s == null) s = "?";
						writer.write("\t" + s);
						}
					log("adni_extract_xml_values: Wrote data for '" + files[i] + "'.");
					success_count++;
				}else{
					log("adni_extract_xml_values: No subject ID for '" + files[i] + "'.");
					fail_count++;
					}
				
				reader.close();
			
			}catch (IOException ex){
				log("adni_extract_xml_values: Error extracting from '" + files[i] + ".");
				fail_count++;
				}
			
			}
		try{
			writer.close();
		}catch (IOException ex){
			log("Error closing output file.");
			}
		
		log("Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
	}
	
	/*************************************************************************
	 * Moves ADNI data into two CBrain input directories, based upon field strength (1.5 and 3T).
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li><code>source_dir</code> - Directory currently holding ADNI data.
	 * <li><code>target_dir_prefix</code> - Prefix for the target directories. _1.5T and _3.0T
	 * will be added.
	 * <li><code>field_strength_file</code> - Tab delimited file associating subject id with field strength
	 * </ul>
	 * 
	 * @return
	 */
	boolean adni_create_input_dirs(){
		
		log("Starting adni_create_input_dirs.");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("adni_create_input_dirs: source_dir is a required parameter.");
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("adni_create_input_dirs: Cannot find source dir: " + source_dir.getAbsolutePath());
			return false;
			}
		
		_param = parameters.get("target_dir_prefix");
		if (_param == null){
			log("adni_create_input_dirs: target_dir_prefix is a required parameter.");
			return false;
			}
		
		File target_dir_15 = new File(_param + "_15T");
		if (!target_dir_15.exists() && !target_dir_15.mkdir()){
			log("adni_create_input_dirs: Cannot create target dir: " + target_dir_15.getAbsolutePath());
			return false;
			}
		
		File target_dir_30 = new File(_param + "_30T");
		if (!target_dir_30.exists() && !target_dir_30.mkdir()){
			log("adni_create_input_dirs: Cannot create target dir: " + target_dir_30.getAbsolutePath());
			return false;
			}
		
		_param = parameters.get("field_strength_file");
		if (_param == null){
			log("adni_create_input_dirs: field_strength_file is a required parameter.");
			return false;
			}
		
		File field_strength_file = new File(_param);
		if (!field_strength_file.exists()){
			log("adni_create_input_dirs: Cannot find field strength file '" + field_strength_file.getAbsolutePath() + "'.");
			return false;
			}
		
		HashMap<String, Boolean> is_15 = new HashMap<String, Boolean>();
		
		try{
			BufferedReader reader = new BufferedReader(new FileReader(field_strength_file));
			
			String line = reader.readLine();
			line = reader.readLine();	//skip header
			
			while (line != null){
				StringTokenizer tokens = new StringTokenizer(line);
				String id = tokens.nextToken();
				Boolean b = Double.valueOf(tokens.nextToken()) < 2;  
				is_15.put(id, b);
				line = reader.readLine();
				}
			
			reader.close();
			
		}catch (IOException ex){
			log("adni_create_input_dirs: Error reading field strength file '" + field_strength_file.getAbsolutePath() + "'.");
			return false;
			}
		
		String[] subjects = source_dir.list(IoFunctions.getDirFilter());
		int success_count = 0, fail_count = 0;
		
		for (int s = 0; s < subjects.length; s++){
			String subject_id = subjects[s];
			
			File subject_dir = new File(source_dir.getAbsolutePath() + File.separator + subject_id);
			String[] images = subject_dir.list(getMincFilter());
			
			boolean success = true;
			for (int i = 0; success && i < images.length; i++){
				success = false;
				String image = images[i];
				String id = image.substring(0, image.indexOf(".mnc"));
				if (!is_15.containsKey(subject_id)){
					log("adni_create_input_dirs: No field strength found for '" + subject_id + "'.");
				}else{
					File image_file = new File(subject_dir.getAbsolutePath() + File.separator + image);
					File target_file = null;
					String target = "";
					if (is_15.get(subject_id)){
						target_file = new File(target_dir_15.getAbsolutePath() + File.separator + image);
						target = target_dir_15.getName();
					}else{
						target_file = new File(target_dir_30.getAbsolutePath() + File.separator + image);
						target = target_dir_30.getName();
						}
					try{
						IoFunctions.copyFile(image_file, target_file);
						
						log("adni_create_input_dirs: Copied '" + image + "' to '" + target + "'.");
						success = true;
					}catch (IOException ex){
						log("adni_create_input_dirs: Error copying '" + image + "' to '" + target + "'.");
						}
					}
				if (success)
					success_count++;
				else
					fail_count++;
				}
			}
		
		log("adni_create_input_dirs: Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
		
	}
	
	
	/*******************************************************************
	 * Searches a directory with subject subdirectories; for each directory, finds two images files:
	 * the pre-scale image and the post-scale image. Runs <code>minc_tracc</code> on these images
	 * to compute the transform that scales the first to the second, and saves this transform as
	 * and xfm file.
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li><code>source_dir</code> - The directory containing subject images
	 * <li><code>target_dir</code> - The directory to store the transform files
	 * <li>{@code param_file} - [optional] Text file which specifies parameters for the minctracc command; 
	 * one line that gets inserted into the command. If unspecified, the parameters are "-lsq9 -clobber -quiet" 
	 * <li><code>prefix</code> [optional] - Common prefix for filenames
	 * <li>{@code suffix} [optional] - Common suffix for filenames
	 * <li><code>pre_pattern</code> [optional if <code>post_pattern</code> is specified] - Filename 
	 * pattern identifying the pre-scale image
	 * <li><code>post_pattern</code> [optional if <code>pre_pattern</code> is specified] - Filename 
	 * pattern identifying the post-scale image
	 * </ul>
	 * 
	 * @return
	 */
	boolean adni_compute_scale_transforms(){
		
		log("Starting adni_compute_scale_transforms.");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("adni_compute_scale_transforms: source_dir is a required parameter.", 
					LoggingType.Errors);
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("adni_compute_scale_transforms: Cannot find source dir: " + source_dir.getAbsolutePath(), 
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("target_dir");
		if (_param == null){
			log("adni_compute_scale_transforms: target_dir is a required parameter.", LoggingType.Errors);
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists() && !target_dir.mkdir()){
			log("adni_compute_scale_transforms: Cannot create target dir: " + target_dir.getAbsolutePath(), 
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("param_file");
		String minctracc_params = " -lsq9 -clobber -quiet ";
		if (_param != null){
			try{
				File param_file = new File(_param);
				BufferedReader reader = new BufferedReader(new FileReader(param_file));
				String line = reader.readLine();
				if (line != null)
					minctracc_params = " " + line + " ";
			}catch (IOException ex){
				log("adni_compute_scale_transforms: Cannot read parameter file: " + _param, LoggingType.Errors);
				return false;
				}
			}
		
		String pre_pattern = parameters.get("pre_pattern");
		String post_pattern = parameters.get("post_pattern");
		
		if (pre_pattern == null && post_pattern == null){
			log("adni_compute_scale_transforms: one or both of pre_pattern or post_pattern are required parameters.", 
					LoggingType.Errors);
			return false;
			}
		
		String prefix = parameters.get("prefix");
		if (prefix == null){ 
			prefix = "";
		}else if (!prefix.endsWith("_")){
			prefix = prefix + "_";
			}
			
		String suffix = parameters.get("suffix");
		if (suffix == null){
			suffix = "";
		}else if (!suffix.endsWith("_")){
			suffix = "_" + suffix;
			}
		
		String[] subjects = source_dir.list(IoFunctions.getDirFilter());
		int success_count = 0, fail_count = 0;
		
		log("adni_compute_scale_transforms: Getting list of subject images...");
		HashMap<String,AdniScaleItem> subject_images = new HashMap<String,AdniScaleItem>();
		
		for (int s = 0; s < subjects.length; s++){
			String subject_id = subjects[s];
			File subject_dir = new File(source_dir.getAbsolutePath() + File.separator + subject_id);
			
			String[] images = subject_dir.list(getMincFilter());
			Arrays.sort(images);
			
			AdniScaleItem item = new AdniScaleItem();
			
			for (int i = 0; i < images.length; i++){
				//Depends on sort order of images
				String this_image = images[i];
				String stripped = this_image.substring(prefix.length() + subject_id.length() + 1);
				int a = stripped.indexOf("_");
				String visit = stripped.substring(0, a);
				item.visits.add(visit); 
				stripped = stripped.substring(a + 1);
				stripped = stripped.substring(0, stripped.indexOf(suffix + ".mnc"));
				if (stripped.equals(pre_pattern)){
					item.pre_images.put(visit, subject_dir.getAbsolutePath() + File.separator + this_image);
				}else if (stripped.equals(post_pattern)){
					item.post_images.put(visit, subject_dir.getAbsolutePath() + File.separator + this_image);
					}
				}
			
			if (item.pre_images.size() == 0 || item.post_images.size() == 0){
				log("adni_compute_scale_transforms: No images found for subject '" + subject_id + "'."); //, 
				//		LoggingType.Errors);
				fail_count++;
			}else if (item.pre_images.size() != item.post_images.size()){
				log("adni_compute_scale_transforms: Unmatched pre- and post-scale images for subject '" + subject_id + "'."); //,
				//		LoggingType.Errors);
				fail_count++;
			}else{
				log("adni_compute_scale_transforms: Added subject '" + subject_id + "'.");
				subject_images.put(subject_id, item);
				}
			}
		
		log("adni_compute_scale_transforms: Computing transforms...");
		
		ArrayList<String> ok_subjects = new ArrayList<String>(subject_images.keySet());
		Collections.sort(ok_subjects);
		for (int s = 0; s < ok_subjects.size(); s++){
			String subject_id = ok_subjects.get(s);
			AdniScaleItem item = subject_images.get(subject_id);
			ArrayList<String> visits = new ArrayList<String>(item.visits);
			
			for (int i = 0; i < visits.size(); i++){
				String visit = visits.get(i);
				String pre_image = item.pre_images.get(visit);
				String post_image = item.post_images.get(visit);
				String output_file = target_dir.getAbsolutePath() + File.separator + prefix + subject_id + "_" + visit + suffix + ".xfm";
				
				String command = "minctracc " + minctracc_params + pre_image + " " + post_image + " " + output_file;
				
				ProcessState state = ProcessState.NotStarted;
				
				try{
					state = CommandFunctions.callAndWait(command);
//					CommandFunctions.call(command);
				}catch (IOException ex){
					state = ProcessState.TerminatedWithError;
					}
				
				if (state == ProcessState.TerminatedNormally){
					log("adni_compute_scale_transforms: Computed transform for subject '" + subject_id + 
									"', visit '" + visit + "'.");
					success_count++;
				}else{
					log("adni_compute_scale_transforms: Failed to compute transform for subject '" + subject_id + 
							"', visit '" + visit + "'.",
							LoggingType.Errors);
					fail_count++;
					}
				
				}
			
			}
		
		log("Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
		
	}
	
	class AdniScaleItem {
		
		public TreeSet<String> visits = new TreeSet<String>();
		public HashMap<String, String> pre_images = new HashMap<String, String>();
		public HashMap<String, String> post_images = new HashMap<String, String>();
		
	}
	
	/********************************************************************
	 * Transforms a directory of minc images using a list of transformation files with identical file names.
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>{@code source_dir} - Directory containing the images to be transformed
	 * <li>{@code target_dir} - Directory to store the transformed images
	 * <li>{@code transform_dir} - Directory containing the transform (xfm) files
	 * <li>{@code param_file} - [optional] Parameter file, containing a line of parameters for the {@code mincresample} command
	 * </ul>
	 * 
	 * @return
	 */
	boolean adni_transform_images(){
		
		log("Starting adni_transform_images.");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("adni_transform_images: source_dir is a required parameter.", 
					LoggingType.Errors);
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("adni_transform_images: Cannot find source dir: " + source_dir.getAbsolutePath(), 
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("transform_dir");
		if (_param == null){
			log("adni_transform_images: transform_dir is a required parameter.", 
					LoggingType.Errors);
			return false;
			}
		
		File transform_dir = new File(_param);
		if (!transform_dir.exists() || !transform_dir.isDirectory()){
			log("adni_transform_images: Cannot find transform dir: " + source_dir.getAbsolutePath(), 
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("target_dir");
		if (_param == null){
			log("adni_transform_images: target_dir is a required parameter.", LoggingType.Errors);
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists() && !target_dir.mkdir()){
			log("adni_transform_images: Cannot create target dir: " + target_dir.getAbsolutePath(), 
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("param_file");
		String mincresample_params = " -tfm_input_sampling -quiet ";
		if (_param != null){
			try{
				File param_file = new File(_param);
				BufferedReader reader = new BufferedReader(new FileReader(param_file));
				String line = reader.readLine();
				if (line != null)
					mincresample_params = " " + line + " ";
			}catch (IOException ex){
				log("adni_transform_images: Cannot read parameter file: " + _param, LoggingType.Errors);
				return false;
				}
			}
		
		// List of transform files
		TreeSet<String> transforms = new TreeSet<String>();
		String[] files = transform_dir.list(getXfmFilter());
		for (int i = 0; i < files.length; i++){
			transforms.add(files[i].substring(0, files[i].lastIndexOf(".")));
			}
		
		// List of images with transforms
		files = source_dir.list(getMincFilter());
		ArrayList<String> images = new ArrayList<String>();
		for (int i = 0; i < files.length; i++){
			String image = files[i].substring(0, files[i].lastIndexOf("."));
			if (transforms.contains(image))
				images.add(image);
			}
		
		// Run resample
		int success_count = 0, fail_count = 0;
		for (int i = 0; i < images.size(); i++){
			String image = images.get(i);
			String command = "mincresample" + mincresample_params + "-transformation " + transform_dir.getAbsolutePath() + 
							 File.separator + image + ".xfm " + source_dir.getAbsolutePath() + File.separator + image +
							 ".mnc " + target_dir.getAbsolutePath() + File.separator + image + ".mnc";
			
			ProcessState state = ProcessState.NotStarted;
			
			try{
				state = CommandFunctions.callAndWait(command);
//				CommandFunctions.call(command);
			}catch (IOException ex){
				state = ProcessState.TerminatedWithError;
				}
			
			if (state == ProcessState.TerminatedNormally){
				log("adni_transform_images: Transformed image '" + image + "'.");
				success_count++;
			}else{
				log("adni_transform_images: Failed to transform image '" + image + "'.", LoggingType.Errors);
				fail_count++;
				}
			
			}
		
		log("Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
	}
	
	/********************************************************************
	 * Searches a tab-delimited validation file and moves all subjects matching <code>valid_pattern</code>
	 * from <code>source_dir</code> to <code>target_dir</code>.
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li><code>source_dir</code> - Directory currently storing the civet output
	 * <li><code>target_dir</code> - Directory in which to store the validated folders
	 * <li><code>validation_file</code> - Tab-delimited text file with subject_id in the first column and 
	 * validation_code in the second.
	 * <li><code>valid_pattern</code> - Pattern to match in the second column (case insensitive).
	 * <li><code>copy</code> [optional] - If specified, subjects are copied rather than moved.
	 * </ul>
	 * 
	 * @return
	 */
	boolean adni_move_valid_subjects(){
		
		log("Starting move_validated_subjects.");
		
		boolean copy = parameters.containsKey("copy");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("move_validated_subjects: source_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("move_validated_subjects: Cannot find source dir: " + source_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("target_dir");
		if (_param == null){
			log("move_validated_subjects: target_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists() && !target_dir.mkdir()){
			log("move_validated_subjects: Cannot create target dir: " + target_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("validation_file");
		if (_param == null){
			log("move_validated_subjects: validation_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File validation_file = new File(_param);
		if (!validation_file.exists()){
			log("move_validated_subjects: Cannot find validation file: " + validation_file.getAbsolutePath());
			return false;
			}
		
		String valid_pattern = parameters.get("valid_pattern");
		
		if (valid_pattern == null){
			log("move_validated_subjects: valid_pattern is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		valid_pattern = valid_pattern.toLowerCase();
		
		TreeSet<String> valid_subjects = new TreeSet<String>();
		try{
			BufferedReader reader = new BufferedReader(new FileReader(validation_file));
			String line = reader.readLine();
			if (line != null)
				line = reader.readLine(); //skip header
			
			while (line != null){
				if (line.length() > 0){
					StringTokenizer tokens = new StringTokenizer(line, "\t");
					String subject_id = tokens.nextToken();
					if (tokens.hasMoreTokens()){
						String valid = tokens.nextToken().toLowerCase();
						if (valid.contains(valid_pattern)){
							valid_subjects.add(subject_id);
							}
						}
					}
				line = reader.readLine();
				}
			
			reader.close();
		}catch (Exception ex){
			log("move_validated_subjects: Error reading validation file: " + validation_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		if (valid_subjects.size() == 0){
			log("move_validated_subjects: No valid subjects found!",
					LoggingType.Errors);
			return false;
			}
		
		String[] subjects = source_dir.list(IoFunctions.getDirFilter());
		int success_count = 0, fail_count = 0;
		
		for (int i = 0; i < subjects.length; i++){
			String subject = subjects[i];
			
			if (valid_subjects.contains(subject)){
				File dir_old = new File(source_dir.getAbsolutePath() + File.separator + subject);
				File dir_new = new File(target_dir.getAbsolutePath() + File.separator + subject);
				
				if (!copy){
					if (dir_new.exists() && !dir_new.delete()){
						log("move_validated_subjects: Could not delete existing " + subject + ".",
								LoggingType.Errors);
						fail_count++;
					}else{
						if (!dir_old.renameTo(dir_new)){
							log("move_validated_subjects: Failed to move " + subject + ".",
									LoggingType.Errors);
							fail_count++;
						}else{
							log("move_validated_subjects: Moved " + subject + ".");
							success_count++;
							}
						}
				}else{
					try{
						if (!IoFunctions.copyDir(dir_old, dir_new)){
							log("move_validated_subjects: Failed to copy " + subject + ".", LoggingType.Errors);
							fail_count++;
						}else{
							log("move_validated_subjects: Copied " + subject + ".");
							success_count++;
							}
					}catch (IOException e){
						log("move_validated_subjects: Failed to copy " + subject + ".", LoggingType.Errors);
						fail_count++;
						}
					}
				}
			}
		
		log("Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
		
	}
	
	/********************************************************************
	 * Searches a tab-delimited validation file and moves all subject images matching <code>valid_pattern</code>
	 * from <code>source_dir</code> to <code>target_dir</code>. The image file name should be formatted as
	 * <code>prefix</code>[_subject_id_]<code>suffix</code>.mnc
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li><code>source_dir</code> - Directory currently storing the civet output
	 * <li><code>target_dir</code> - Directory in which to store the validated folders
	 * <li><code>validation_file</code> - Tab-delimited text file with subject_id in the first column and 
	 * validation_code in the second.
	 * <li><code>valid_pattern</code> - Pattern to match in the second column (case insensitive).
	 * <li><code>copy</code> [optional] - If specified, images are copied rather than moved.
	 * <li><code>prefix</code> [optional] - Prefix to remove from file name to create subject ID.
	 * <li><code>suffix</code> [optional] - Suffix to remove from file name to create subject ID.
	 * </ul>
	 * 
	 * @return
	 */
	boolean adni_move_valid_images(){
		
		log("Starting adni_move_valid_images.");
		
		boolean copy = parameters.containsKey("copy");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("adni_move_valid_images: source_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("adni_move_valid_images: Cannot find source dir: " + source_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("target_dir");
		if (_param == null){
			log("adni_move_valid_images: target_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists() && !target_dir.mkdir()){
			log("adni_move_valid_images: Cannot create target dir: " + target_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("validation_file");
		if (_param == null){
			log("adni_move_valid_images: validation_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File validation_file = new File(_param);
		if (!validation_file.exists()){
			log("adni_move_valid_images: Cannot find validation file: " + validation_file.getAbsolutePath());
			return false;
			}
		
		String valid_pattern = parameters.get("valid_pattern");
		
		if (valid_pattern == null){
			log("adni_move_valid_images: valid_pattern is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		valid_pattern = valid_pattern.toLowerCase();
		
		String prefix = parameters.get("prefix");
		if (!prefix.endsWith("_"))
			prefix = prefix + "_";
		String suffix = parameters.get("suffix");
		if (!suffix.endsWith("_"))
			suffix = "_" + suffix;
		
		TreeSet<String> valid_subjects = new TreeSet<String>();
		try{
			BufferedReader reader = new BufferedReader(new FileReader(validation_file));
			String line = reader.readLine();
			if (line != null)
				line = reader.readLine(); //skip header
			
			while (line != null){
				if (line.length() > 0){
					StringTokenizer tokens = new StringTokenizer(line, "\t");
					String subject_id = tokens.nextToken();
					if (tokens.hasMoreTokens()){
						String valid = tokens.nextToken().toLowerCase();
						if (valid.contains(valid_pattern)){
							valid_subjects.add(subject_id);
							}
						}
					}
				line = reader.readLine();
				}
			
			reader.close();
		}catch (Exception ex){
			log("adni_move_valid_images: Error reading validation file: " + validation_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		if (valid_subjects.size() == 0){
			log("adni_move_valid_images: No valid subjects found!",
					LoggingType.Errors);
			return false;
			}
		
		String[] images = source_dir.list(getMincFilter());
		int success_count = 0, fail_count = 0;
		
		for (int i = 0; i < images.length; i++){
			String image = images[i];
			String subject_id = image.substring(0, image.lastIndexOf(".mnc"));
			if (prefix != null)
				subject_id = subject_id.substring(prefix.length());
			if (suffix != null)
				subject_id = subject_id.substring(0, subject_id.lastIndexOf(suffix));
			
			if (valid_subjects.contains(subject_id)){
				File file_old = new File(source_dir.getAbsolutePath() + File.separator + image);
				File file_new = new File(target_dir.getAbsolutePath() + File.separator + image);
				
				if (!copy){
					if (!file_old.renameTo(file_new)){
						log("adni_move_valid_images: Failed to move " + image + ".",
								LoggingType.Errors);
						fail_count++;
					}else{
						log("adni_move_valid_images: Moved " + image + ".");
						success_count++;
						}
				}else{
					try{
						if (!IoFunctions.copyFile(file_old, file_new)){
							log("adni_move_valid_images: Failed to copy " + image + ".",
									LoggingType.Errors);
							fail_count++;
						}else{
							log("adni_move_valid_images: Copied " + image + ".");
							success_count++;
							}
					}catch (IOException e){
						log("adni_move_valid_images: Failed to copy " + image + ".",
								LoggingType.Errors);
						fail_count++;
						}
					}
				}
			}
		
		log("Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
		
	}
	
	/********************************************************
	 * Modifies CBrain Civet output by stripping the folder names to leave only the subject id;
	 * moves to <code>target_dir</code>.
	 * 
	 * @return
	 */
	boolean adni_strip_cbrain_names(){
		
		//Rename files to remove server info
		//File dir = new File("/export02/data/adni_civet_output_final");
		//File dir2 = new File("/export02/data/adni_civet_output_repeats");
		
		log("Starting adni_strip_cbrain_names.");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("adni_strip_cbrain_names: source_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("adni_strip_cbrain_names: Cannot find source dir: " + source_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("target_dir");
		if (_param == null){
			log("adni_strip_cbrain_names: target_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists() && !target_dir.mkdir()){
			log("adni_strip_cbrain_names: Cannot create target dir: " + target_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		String[] files = source_dir.list(IoFunctions.getDirFilter());
		int success_count = 0, fail_count = 0;
		
		for (int i = 0; i < files.length; i++){
			File file = new File(source_dir.getAbsolutePath() + File.separator + files[i]);
			if (file.isDirectory()){
				String name = file.getName();
				String old_name = file.getName();
				if (name.contains("-Mammouth")){
					name = name.substring(0, name.indexOf("-Mammouth"));
					File new_file = new File(target_dir.getAbsolutePath() + File.separator + name);
					if (file.renameTo(new_file)){
						System.out.println(old_name + " -> " + new_file.getName());
						success_count++;
					}else{
						System.out.println(old_name + " could not be renamed.");
						fail_count++;
						}
				}else if (name.contains("-Colosse")){
					name = name.substring(0, name.indexOf("-Colosse"));
					File new_file = new File(target_dir.getAbsolutePath() + File.separator + name);
					if (file.renameTo(new_file)){
						System.out.println(old_name + " -> " + new_file.getName());
						success_count++;
					}else{
						name = target_dir.getAbsolutePath() + File.separator + file.getName();
						if (file.renameTo(new File(name))){
							System.out.println(old_name + " -> " + target_dir.getName());
							success_count++;
						}else{
							System.out.println(old_name + " could not be renamed.");
							fail_count++;
							}
						}
					}
				}
			}
		
		log("Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
		
	}
	
	/**********************************************************
	 * Generates a report from a tab-delimited text validation file. The file should have at least two columns, the first containing
	 * subject IDs, and the others containing validity information.
	 * 
	 * <p>The report will contain frequencies of occurrence for specific elements, identified by a definition file. The
	 * definition file must be a text file with n sections, where each section is preceded by the tag {@code <section_name>}. Within 
	 * these sections, each line designates a tag to extract a frequency for, and, optionally, a string to use in the report. The 
	 * second must be designated by an "=" sign; if it is not specified, the tag itself will be reported. For example, the following 
	 * tag counts all occurrences of the string "rmt" and reports them as "Right medial temporal":
	 * 
	 * <p>{@code rmt = Right medial temporal }
	 * 
	 * <p>A levels file can optionally be specified. This is a two-column tab-delimited text file with subject id in the first column
	 * and associated levels in the second. If specified, the report will be split into as many subsections as there are unique levels,
	 * and stats for these levels will be reported in addition to the overall stats. The {@code has_header} flag determines whether the
	 * first line will be skipped or not.
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>{@code validation_file} - Tab-delimited validation file
	 * <li>{@code definition_file} - Text file defining the report
	 * <li>{@code output_file} - File in which to write the report
	 * <li>{@code levels_file} - [optional] - If specified, splits cohort into levels defined by this tab-delimited text file
	 * <li>{@code has_header} [optional] - If specified, the first line of the validation/levels files will be skipped
	 * <li>{@code as_table} [optional] - If specified, output will be formatted as a tab-delimited table
	 * </ul>
	 * 
	 * @return {@code true} if successful.
	 */
	boolean adni_validation_report(){
		
		log("Starting adni_validation_report.");
		
		String _param = parameters.get("validation_file");
		if (_param == null){
			log("adni_validation_report: validation_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File validation_file = new File(_param);
		if (!validation_file.exists()){
			log("adni_validation_report: Cannot find validation file: " + validation_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("definition_file");
		if (_param == null){
			log("adni_validation_report: definition_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File definition_file = new File(_param);
		if (!definition_file.exists()){
			log("adni_validation_report: Cannot find definition file: " + definition_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("output_file");
		if (_param == null){
			log("adni_validation_report: output_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File output_file = new File(_param);
		if (output_file.exists() && !output_file.delete()){
			log("adni_validation_report: Cannot delete existing output file: " + output_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("levels_file");
		File levels_file = null;
		
		if (_param != null){
			levels_file = new File(_param);
			if (!levels_file.exists()){
				log("adni_validation_report: Cannot find levels file: " + levels_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
		
		boolean has_header = parameters.containsKey("has_header");
		boolean as_table = parameters.containsKey("as_table");
		
		try{
			HashMap<String,AdniStats> level_stats = new HashMap<String,AdniStats>();
			HashMap<String,String> subject_levels = null;
			HashMap<String,HashMap<String,String>> tag_map = new HashMap<String,HashMap<String,String>>();
			ArrayList<String> sections = new ArrayList<String>();
			
			//Add all subjects to the overall level
			AdniStats overall_stats = new AdniStats();
			//level_stats.put("Overall", overall_stats);
			
			//read levels if specified
			if (levels_file != null){
				BufferedReader reader = new BufferedReader(new FileReader(levels_file));
				//get list of unique levels
				subject_levels = new HashMap<String,String>();
				
				String line = reader.readLine();
				if (has_header) line = reader.readLine();
				
				while (line != null){
					StringTokenizer tokens = new StringTokenizer(line);
					String id = tokens.nextToken();
					String level = tokens.nextToken();
					
					subject_levels.put(id, level);
					//overall_stats.subjects.add(id);
					if (!level_stats.containsKey(level)){
						AdniStats stats = new AdniStats();
						stats.subjects.add(id);
						level_stats.put(level, stats);
					}else{
						AdniStats stats = level_stats.get(level);
						stats.subjects.add(id);
						}
					
					line = reader.readLine();
					}
				
				}
			
			ArrayList<String> levels = new ArrayList<String>(level_stats.keySet());
			Collections.sort(levels);
			
			BufferedReader reader = new BufferedReader(new FileReader(definition_file));
			
			String line = reader.readLine();
			String current_wrap = null;
			int l = 1;
			HashMap<String, String> current_map = null;
			boolean is_wrap;
			
			// Read definition file
			while (line != null){
				is_wrap = false;
				if (line.length() > 0){
					if (line.contains("<")){
						int a = line.indexOf("<") + 1;
						int b = line.indexOf(">");
						if (a >= b){
							log("adni_validation_report: Error parsing line " + l + ": " + line, LoggingType.Errors);
							return false;
							}
						current_wrap = line.substring(a, b);
						current_map = new HashMap<String, String>();
						tag_map.put(current_wrap, current_map);
						sections.add(current_wrap);
						
						overall_stats.frequency_map.put(current_wrap, new HashMap<String, Integer>());
						
						//add frequency map for each level
						for (int i = 0; i < levels.size(); i++){
							AdniStats stats = level_stats.get(levels.get(i));
							stats.frequency_map.put(current_wrap, new HashMap<String, Integer>());
							}
						is_wrap = true;
						}
					
					if (!is_wrap && current_map != null){
						if (!line.contains("=")){
							current_map.put(line, line);
						}else{
							String[] parts = line.split("=");
							String tag = parts[0];
							String full = parts[1];
							if (tag.length() == 0 || full.length() == 0){
								log("adni_validation_report: Error parsing line " + l + ": " + line, LoggingType.Errors);
								return false;
								}
							current_map.put(tag, full);
							}
						}
					}
				line = reader.readLine();
				}
			
			reader.close();
			
			// Read validation file
			reader = new BufferedReader(new FileReader(validation_file));
			line = reader.readLine();
			if (has_header) line = reader.readLine();
			
			int total_ids = 0;
			
			while (line != null){
				StringTokenizer tokens = new StringTokenizer(line, "\t");
				int count = tokens.countTokens();
				
				if (count > 0){
					total_ids++;
					String id = tokens.nextToken();
					int i = 1;
					
					// For overall stats
					HashMap<String,HashMap<String,Integer>> frequency_map = overall_stats.frequency_map;
					//HashMap<String,Integer> counts = stats.counts;
					while (i < count && i <= frequency_map.size()){
						String section = sections.get(i - 1);
						HashMap<String,Integer> counts = frequency_map.get(section);
						ArrayList<String> tags = new ArrayList<String>(tag_map.get(section).keySet());
						String cell = tokens.nextToken();
						for (int t = 0; t < tags.size(); t++){
							String tag = tags.get(t);
							if (cell.contains(tag)){
								int c = 0;
								if (counts.containsKey(tag))
									c = counts.get(tag);
								counts.put(tag, c + 1);
								}
							}
						i++;
						}
					
					// For each level
					for (int level = 0; level < levels.size(); level++){
						AdniStats stats = level_stats.get(levels.get(level));
						i = 1;
						tokens = new StringTokenizer(line, "\t");
						id = tokens.nextToken();
						if (stats.subjects.contains(id)){
							stats.subjects_in_list++;
							frequency_map = stats.frequency_map;
							//HashMap<String,Integer> counts = stats.counts;
							while (i < count && i <= frequency_map.size()){
								String section = sections.get(i - 1);
								HashMap<String,Integer> counts = frequency_map.get(section);
								ArrayList<String> tags = new ArrayList<String>(tag_map.get(section).keySet());
								String cell = tokens.nextToken();
								for (int t = 0; t < tags.size(); t++){
									String tag = tags.get(t);
									if (cell.contains(tag)){
										int c = 0;
										if (counts.containsKey(tag))
											c = counts.get(tag);
										counts.put(tag, c + 1);
										}
									}
								i++;
								}
							}
						}
					}
				
				line = reader.readLine();
				}
			
			reader.close();
			
			//Write frequencies to report
			BufferedWriter writer = new BufferedWriter(new FileWriter(output_file));
			
			if (!as_table){
			
				writer.write("ADNI Report - Frequencies from validation file");
				
				writer.write("\n\nOverall results:");
				writer.write("\n\nTotal subjects: " + total_ids);
				
				//Overall stats
				for (int i = 0; i < sections.size(); i++){
					String section = sections.get(i);
					writer.write("\n\n" + sections.get(i));
					
					HashMap<String,String> titles = tag_map.get(section);
					HashMap<String,Integer> frequencies = overall_stats.frequency_map.get(section);
					
					ArrayList<String> tags = new ArrayList<String>(titles.keySet());
					Collections.sort(tags);
					
					for (int j = 0; j < tags.size(); j++){
						String tag = tags.get(j);
						String title = titles.get(tag);
						int freq = frequencies.get(tag);
						
						double pct = (double)freq / (double)total_ids;
						String pct_str = MguiDouble.getString(pct * 100.0, "#0") + "%";
						
						writer.write("\n" + title + ": " + freq + " (" + pct_str + ")");
						}
					
					}
				
				//Subsection for each level
				for (int level = 0; level < levels.size(); level++){
					AdniStats stats = level_stats.get(levels.get(level));
					writer.write("\n\nLevel: " + levels.get(level));
					int total = stats.subjects_in_list;
					writer.write("\n\nTotal subjects: " + total);
					for (int i = 0; i < sections.size(); i++){
						String section = sections.get(i);
						writer.write("\n\n" + sections.get(i));
						
						HashMap<String,String> titles = tag_map.get(section);
						HashMap<String,Integer> frequencies = stats.frequency_map.get(section);
						
						ArrayList<String> tags = new ArrayList<String>(titles.keySet());
						Collections.sort(tags);
						
						for (int j = 0; j < tags.size(); j++){
							String tag = tags.get(j);
							String title = titles.get(tag);
							int freq = 0;
							
							if (frequencies.containsKey(tag))
								freq = frequencies.get(tag);
							
							double pct = (double)freq / (double)total;
							String pct_str = MguiDouble.getString(pct * 100.0, "#0") + "%";
							
							writer.write("\n" + title + ": " + freq + " (" + pct_str + ")");
							}
						
						}
					}
			}else{
				
				//header is levels
				writer.write("Level\tTotal subjects\tSection\tTag\tFrequency\tPct");
				//writer.write("Section\tTag\tOverall\tOverall(%)");
				//for (int i = 0; i < levels.size(); i++)
				//	writer.write("\t" + levels.get(i) + "\t" + levels.get(i) + "(%)");
								
				//totals
				//writer.write("\n\tTotals\t" + total_ids);
				//for (int i = 0; i < levels.size(); i++)
				//	writer.write("\t" + level_stats.get(levels.get(i)).subjects_in_list);
				
				
				for (int i = 0; i < sections.size(); i++){
					String section = sections.get(i);
					//writer.write("\n" + section);
					
					HashMap<String,String> titles = tag_map.get(section);
					
					ArrayList<String> tags = new ArrayList<String>(titles.keySet());
					Collections.sort(tags);
					
					for (int j = 0; j < tags.size(); j++){
						String tag = tags.get(j);
						String title = titles.get(tag);
						//writer.write("\nOverall\t" + total_ids + "\t" + title);
						
						//overall
						HashMap<String,Integer> frequencies = overall_stats.frequency_map.get(section);
						int freq = frequencies.get(tag);
						
						double pct = (double)freq / (double)total_ids;
						String pct_str = MguiDouble.getString(pct * 100.0, "#0") + "%";
						//writer.write("\t" + freq + "\t" + pct_str);
						
						writer.write("\nOverall\t" + total_ids + "\t" + section + "\t" + title + "\t" + freq + "\t" + pct_str);
						}
					}
				
				for (int level = 0; level < levels.size(); level++){
					AdniStats stats = level_stats.get(levels.get(level));
					//writer.write("\n" + levels.get(level));
					int total = stats.subjects_in_list;
					//writer.write("\t" + total);
					for (int i = 0; i < sections.size(); i++){
						String section = sections.get(i);
						//writer.write("\t" + sections.get(i));
						
						HashMap<String,String> titles = tag_map.get(section);
						HashMap<String,Integer> frequencies = stats.frequency_map.get(section);
						
						ArrayList<String> tags = new ArrayList<String>(titles.keySet());
						Collections.sort(tags);
						
						for (int j = 0; j < tags.size(); j++){
							String tag = tags.get(j);
							String title = titles.get(tag);
							int freq = 0;
							
							if (frequencies.containsKey(tag))
								freq = frequencies.get(tag);
							
							double pct = (double)freq / (double)total;
							String pct_str = MguiDouble.getString(pct * 100.0, "#0") + "%";
							
							writer.write("\n" + levels.get(level) + "\t" + total + "\t" + section + "\t" + title + "\t" + freq + "\t" + pct_str);
							}
						}
					}
				}
				
			writer.close();
			
		}catch (IOException ex){
			log("adni_validation_report: IOException encountered.", LoggingType.Errors);
			return false;
			}
		
		log("Done. Written to '" + output_file + "'.");
		
		return true;
	}
	
	/*********************************************************************
	 * Searches Civet "log" directories and counts the number of specific failures.
	 * 
	 * <p>Parameters
	 * <ul>
	 * <li>{@code source_dir} - Directory containing Civet output
	 * <li>{@code output_file} - File in which to store the results (tab-delimited text)
	 * </ul>
	 * 
	 * @return
	 */
	boolean civet_failures(){
		
		log("Starting civet_failures.");
		
		String _param = parameters.get("source_dir");
		if (_param == null){
			log("civet_failures: source_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File source_dir = new File(_param);
		if (!source_dir.exists() || !source_dir.isDirectory()){
			log("civet_failures: Cannot find source dir: " + source_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("output_file");
		if (_param == null){
			log("civet_failures: output_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File output_file = new File(_param);
		if (output_file.exists() && !output_file.delete()){
			log("civet_failures: Cannot delete existing output file: " + output_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		String[] subjects = source_dir.list(IoFunctions.getDirFilter());
		if (subjects.length == 0){
			log("civet_failures: No subjects in source_dir", LoggingType.Errors);
			return false;
			}
		
		int success_count = 0, fail_count = 0;
		
		HashMap<String,Integer> counts = new HashMap<String,Integer>();
		
		for (int i = 0; i < subjects.length; i++){
			String subject = subjects[i];
			
			File log_dir = new File(source_dir.getAbsolutePath() + File.separator + subject + File.separator + "logs");
			
			if (!log_dir.exists()){
				log("civet_failures: No log directory for subject '" + subject + "'.", LoggingType.Errors);
				fail_count++;
			}else{
				String[] failures = log_dir.list(getFailedFilter());
				
				if (failures.length > 0){
					for (int f = 0; f < failures.length; f++){
						String item = failures[f];
						item = item.substring(item.indexOf(".") + 1, item.indexOf(".failed"));
						if (!counts.containsKey(item)){
							counts.put(item, 1);
						}else{
							counts.put(item, counts.get(item) + 1);
							}
						}
				}else if (log_dir.list().length == 0){
					String item = "no_logs";
					if (!counts.containsKey(item)){
						counts.put(item, 1);
					}else{
						counts.put(item, counts.get(item) + 1);
						}
					}
				
				log("civet_failures: Processed subject '" + subject + "'.");
				success_count++;
				}
			
			}
		
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(output_file));
		
			ArrayList<String> items = new ArrayList<String>(counts.keySet());
			Collections.sort(items);
			
			writer.write("Civet failure counts for directory " + source_dir.getName());
			
			for (int i = 0; i < items.size(); i++){
				String item = items.get(i);
				writer.write("\n" + item + "\t" + counts.get(item));
				}
			
			writer.close();
		
		}catch (IOException ex){
			log("civet_failures: Could not write results to '" + output_file.getAbsolutePath() + "'.", LoggingType.Errors);
			return false;
			}
		
		log("civet_failures: Wrote results to '" + output_file.getAbsolutePath() + "'.");
		log("civet_failures: Done. " + success_count + " successes. " + fail_count + " failures.");
		
		
		return fail_count == 0;
	}
	
	
	/*********************************************************
	 * Searches a Civet output directory for subjects with no log files, and moves/copies their native images to 
	 * a new directory. These instances are most likely server failures, rather than Civet failures, so they should be rerun. 
	 * 
	 * <p>Parameters
	 * <ul>
	 * <li>{@code civet_dir} - Directory containing Civet output
	 * <li>{@code image_dir} - Directory containing native images
	 * <li>{@code target_dir} - Directory in which to store moved/copied images
	 * <li>{@code prefix} [optional] - Prefix for image filenames
	 * <li>{@code suffix} [optional] - Suffix for image filenames
	 * <li>{@code copy} [optional] - If specified, copies the image files; otherwise moves them
	 * </ul>
	 * 
	 * @return
	 */
	boolean civet_move_no_logs(){
		
		log("Starting civet_move_no_logs.");
		
		boolean copy = parameters.containsKey("copy");
		String prefix = "";
		if (parameters.containsKey("prefix")){
			prefix = parameters.get("prefix");
			if (!prefix.endsWith("_")) prefix = prefix + "_";
			}
		String suffix = "";
		if (parameters.containsKey("suffix"))
			suffix = parameters.get("suffix");
		if (!suffix.startsWith("_")) suffix = "_" + suffix;
		
		String _param = parameters.get("civet_dir");
		if (_param == null){
			log("civet_move_no_logs: civet_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File civet_dir = new File(_param);
		if (!civet_dir.exists() || !civet_dir.isDirectory()){
			log("civet_move_no_logs: Cannot find civet_dir: " + civet_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("image_dir");
		if (_param == null){
			log("civet_move_no_logs: source_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File image_dir = new File(_param);
		if (!image_dir.exists() || !image_dir.isDirectory()){
			log("civet_move_no_logs: Cannot find image_dir: " + image_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("target_dir");
		if (_param == null){
			log("civet_move_no_logs: target_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File target_dir = new File(_param);
		if (!target_dir.exists() && !target_dir.mkdir()){
			log("civet_move_no_logs: Cannot find/create target_dir: " + target_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		//List of subjects
		String[] subjects = civet_dir.list(IoFunctions.getDirFilter());
		if (subjects.length == 0){
			log("civet_move_no_logs: No subjects found in " + civet_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		String[] images = image_dir.list(getMincFilter());
		if (images.length == 0){
			log("civet_move_no_logs: No images found in " + image_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		int success_count = 0, fail_count = 0;
		
		for (int i = 0; i < subjects.length; i++){
			String subject = subjects[i];
			File image = new File(image_dir.getAbsolutePath() + File.separator + prefix + subject + suffix + ".mnc");
			File log_dir = new File(civet_dir.getAbsolutePath() + File.separator + subject + File.separator + "logs");
			boolean go = !log_dir.exists();
			
			if (!go){
				String[] contents = log_dir.list();
				go = (contents.length == 0);
				}
			
			if (go){
				String action = "moving";
				if (copy) action = "copying";
				if (!image.exists()){
					log("civet_move_no_logs: No image found for subject '" + subject + "'.", LoggingType.Errors);
					fail_count++;
				}else{
					try{
						File new_image = new File(target_dir.getAbsolutePath() + File.separator + image.getName());
						if (copy){
							IoFunctions.copyFile(image, new_image);
						}else{
							image.renameTo(new_image);
							}
						log("civet_move_no_logs: Success " + action + " image for subject '" + subject + "'.");
						success_count++;
					}catch (IOException ex){
						log("civet_move_no_logs: Error " + action + " image for subject '" + subject + "'.", LoggingType.Errors);
						fail_count++;
						}
					}
				}
			}
		
		log("civet_move_no_logs: Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return fail_count == 0;
	}
	
	/***************************************************************************
	 * Compares the specified surfaces in {@code civet_dir} for self and surface-surface intersections and compiles
	 * a summary report. Can optionally:
	 * 
	 * <ul>
	 * <li>Create a node list for each subject-visit surface (counting the number of associated intersecting faces)
	 * <li>Create a frequency node list for the entire population
	 * <li>Create an ROI-wise frequency report, given a parcellation file
	 * </ul>
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>{@code civet_dir} - The Civet output directory
	 * <li>{@code report_file} - File for the summary report
	 * <li>{@code surface_pattern_1} - Filename pattern identifying the first surface to compare (ends with, 
	 * 								   including file extension)
	 * <li>{@code surface_pattern_2} - Filename pattern identifying the second surface to compare (ends with, 
	 * 								   including file extension)
	 * <li>{@code no_self} [Optional] - Don't check for self intersections
	 * <li>{@code no_surf} [Optional] - Don't check for surf-surf intersections
	 * <li>{@code search_max} [Optional] - Maximum search radius for surface comparison, in mm [default is 10]
	 * <li>{@code subject_report_file} [Optional] - File for subject-wise table report
	 * <li>{@code levels_file} [Optional] - Tab-delimited text file specifying a set of levels to break up the summary; 
	 * 										format is [subject_id]\t[level]
	 * <li>{@code roi_report_file} [Optional] - File for ROI-wise table report 
	 * <li>{@code name_1} [Optional] - Name to identify the first surface (simply "1" if unspecified)
	 * <li>{@code name_2} [Optional] - Name to identify the first surface (simply "2" if unspecified)
	 * <li>{@code roi_file} [Optional] - Vertex-wise ROI assignments (text file, one integer per line)
	 * <li>{@code roi_name_map} [Optional] - Name map for ROIs
	 * <li>{@code frequency_file_pattern} [Optional] - Filename pattern to store vertex-wise frequencies for intersecting faces; will
	 * 										           append "_{name_1}_self.txt", "_{name_2}_self.txt", 
	 * 										           "_{name_1}_{name_2}_surf.txt", and "_{name_2}_{name_1}_surf.txt" to this.
	 * <li>{@code subject_intersection_file_pattern} [Optional]  - Filename pattern to store intersection vertices for each individual
	 * 															   subject. Files will be written to a subdirectory called
	 * 															   "intersections".
	 * <li>{@code subject_list} [Optional] - Operates only on subjects listed in a text file 
	 * </ul>
	 * 
	 * @return
	 */
	boolean surface_intersections_report(){
		
		log("Starting surface_intersections_report.");
		
		boolean no_self = parameters.containsKey("no_self");
		boolean no_surf = parameters.containsKey("no_surf");
		if (no_self && no_surf){
			log("surface_intersections_report: both no_self and no_surf indicated... nothing to do!", LoggingType.Errors);
			return false;
			}
		
		String _param = parameters.get("civet_dir");
		if (_param == null){
			log("surface_intersections_report: civet_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File civet_dir = new File(_param);
		if (!civet_dir.exists() || !civet_dir.isDirectory()){
			log("surface_intersections_report: Cannot find civet_dir: " + civet_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("report_file");
		if (_param == null){
			log("surface_intersections_report: report_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File report_file = new File(_param);
		if (report_file.exists() && !report_file.delete()){
			log("surface_intersections_report: Cannot overwrite report_file: " + report_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		HashMap<String,String> levels_map = null;
		HashMap<String,SurfaceIntersectionStats> levels_frequencies = null;
		_param = parameters.get("levels_file");
		if (_param != null){
			File levels_file = new File(_param);
			if (!levels_file.exists()){
				log("surface_intersections_report: Cannot find levels_file: " + levels_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			levels_map = new HashMap<String,String>();
			levels_frequencies = new HashMap<String,SurfaceIntersectionStats>();
			try{
				BufferedReader reader = new BufferedReader(new FileReader(levels_file));
				String line = reader.readLine();
				while (line != null){
					String[] parts = line.split("\t");
					levels_map.put(parts[0], parts[1]);
					if (!levels_frequencies.containsKey(parts[1]))
						levels_frequencies.put(parts[1], new SurfaceIntersectionStats());
					line = reader.readLine();
					}
			}catch (IOException e){
				log("surface_intersections_report: Error reading levels_file: " + levels_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
		
		// Level sizes
		HashMap<String,MguiInteger> level_sizes = null;
		ArrayList<String> levels = null;
		if (levels_map != null){
			level_sizes = new HashMap<String,MguiInteger>();
			levels = new ArrayList<String>(levels_frequencies.keySet());
			Collections.sort(levels);
			for (int i = 0; i < levels.size(); i++){
				level_sizes.put(levels.get(i), new MguiInteger(0));
				}
			}
		
		String name_1 = parameters.get("name_1");
		if (name_1 == null) name_1 = "1";
		String name_2 = parameters.get("name_2");
		if (name_2 == null) name_2 = "1";
		
		File frequency_file_self_1 = null, frequency_file_self_2 = null; 
		File frequency_file_surf_1 = null, frequency_file_surf_2 = null;
		
		HashMap<File, HashMap<String,File>> frequency_files_levels = null; 
		
		_param = parameters.get("frequency_file_pattern");
		
		if (_param != null){
			if (!no_self){
				frequency_file_self_1 = new File(_param + "_" + name_1 + "_self.txt");
				if (frequency_file_self_1.exists() && !frequency_file_self_1.delete()){
					log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file_self_1.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
				frequency_file_self_2 = new File(_param + "_" + name_2 + "_self.txt");
				if (frequency_file_self_2.exists() && !frequency_file_self_2.delete()){
					log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file_self_2.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
				}
			
			if (!no_surf){
				frequency_file_surf_1 = new File(_param + "_" + name_1 + "_" + name_2 + "_surf.txt");
				if (frequency_file_surf_1.exists() && !frequency_file_surf_1.delete()){
					log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file_surf_1.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
				frequency_file_surf_2 = new File(_param + "_" + name_2 + "_" + name_1 + "_surf.txt");
				if (frequency_file_surf_2.exists() && !frequency_file_surf_2.delete()){
					log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file_surf_2.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
				}
			
			//Vertex-wise level frequencies
			if (levels != null){
				frequency_files_levels = new HashMap<File, HashMap<String, File>>();
				if (!no_self){
					frequency_files_levels.put(frequency_file_self_1, new HashMap<String,File>());
					frequency_files_levels.put(frequency_file_self_2, new HashMap<String,File>());
					}
				if (!no_surf){
					frequency_files_levels.put(frequency_file_surf_1, new HashMap<String,File>());
					frequency_files_levels.put(frequency_file_surf_2, new HashMap<String,File>());
					}
				
				for (int i = 0; i < levels.size(); i++){
					String level = levels.get(i);
					File frequency_file;
					
					if (!no_self){
						frequency_file = new File(_param + "_" + name_1 + "_self_" + level + ".txt");
						if (frequency_file.exists() && !frequency_file.delete()){
							log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file.getAbsolutePath(),
									LoggingType.Errors);
							return false;
							}
						frequency_files_levels.get(frequency_file_self_1).put(level, frequency_file);
						frequency_file = new File(_param + "_" + name_2 + "_self_" + level + ".txt");
						if (frequency_file.exists() && !frequency_file.delete()){
							log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file.getAbsolutePath(),
									LoggingType.Errors);
							return false;
							}
						frequency_files_levels.get(frequency_file_self_2).put(level, frequency_file);
						}
					
					if (!no_surf){
						frequency_file = new File(_param + "_" + name_1 + "_" + name_2 + "_surf_" + level + ".txt");
						if (frequency_file.exists() && !frequency_file.delete()){
							log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file.getAbsolutePath(),
									LoggingType.Errors);
							return false;
							}
						frequency_files_levels.get(frequency_file_surf_1).put(level, frequency_file);
						frequency_file = new File(_param + "_" + name_2 + "_" + name_1 + "_surf_" + level + ".txt");
						if (frequency_file.exists() && !frequency_file.delete()){
							log("surface_intersections_report: Cannot overwrite frequency_file: " + frequency_file.getAbsolutePath(),
									LoggingType.Errors);
							return false;
							}
						frequency_files_levels.get(frequency_file_surf_2).put(level, frequency_file);
						}
					}
				
				}
			}
		
		String subject_intersection_file_pattern = parameters.get("subject_intersection_file_pattern");
		
		File subject_report_file = null;
		_param = parameters.get("subject_report_file");
		if (_param != null){
			subject_report_file = new File(_param);
			if (subject_report_file.exists() && !subject_report_file.delete()){
				log("surface_intersections_report: Cannot overwrite subject_report_file: " + subject_report_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
		
		File roi_report_file = null;
		_param = parameters.get("roi_report_file");
		if (_param != null){
			roi_report_file = new File(_param);
			if (roi_report_file.exists() && !roi_report_file.delete()){
				log("surface_intersections_report: Cannot overwrite roi_report_file: " + roi_report_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
		
		String surface_pattern_1 = parameters.get("surface_pattern_1");
		if (surface_pattern_1 == null){
			log("surface_intersections_report: surface_pattern_1 is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		String surface_pattern_2 = parameters.get("surface_pattern_2");
		if (surface_pattern_2 == null){
			log("surface_intersections_report: surface_pattern_2 is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		int search_max = 5;
		_param = parameters.get("search_max");
		if (_param != null){
			try{
				search_max = Integer.valueOf(_param);
			}catch(NumberFormatException e){
				log("surface_intersections_report: Bad value for search_max: " + _param + ".", LoggingType.Errors);
				return false;
				}
			}
		
		ArrayList<MguiNumber> rois = null;
		_param = parameters.get("roi_file");
		if (_param != null){
			File roi_file = new File(_param);
			if (!roi_file.exists()){
				log("surface_intersections_report: cannot find ROI file: " + roi_file + ".",
						LoggingType.Errors);
				return false;
				}
			MincSurfaceDataLoader loader = new MincSurfaceDataLoader(roi_file, MincSurfaceDataLoader.FORMAT_INT);
			rois = loader.loadValues();
			
			}
		
		NameMap roi_name_map = null;
		
		_param = parameters.get("roi_name_map");
		if (_param != null){
			File roi_name_map_file = new File(_param);
			if (!roi_name_map_file.exists()){
				log("surface_intersections_report: cannot find ROI name map file: " + roi_name_map_file + ".",
						LoggingType.Errors);
				return false;
				}
			NameMapLoader loader = new NameMapLoader(roi_name_map_file);
			
			roi_name_map = new NameMap();
			
			if (!loader.loadNameMap(roi_name_map)){
				log("surface_intersections_report: could not load ROI name map from file: " + roi_name_map_file + ".",
						LoggingType.Errors);
				return false;
				}
		}else if (rois != null){
			// List of ROIS is integers
			roi_name_map = new NameMap();
			
			for (int i = 0; i < rois.size(); i++){
				int roi = (int)rois.get(i).getValue();
				if (!roi_name_map.contains(roi))
					roi_name_map.add(roi, "" + roi);
				}
			}
		
		// ROI sizes
		HashMap<String,MguiInteger> roi_sizes = null;
		if (roi_name_map != null){
			roi_sizes = new HashMap<String,MguiInteger>();
			ArrayList<String> names = new ArrayList<String>(roi_name_map.getNames());
			for (int i = 0; i < names.size(); i++){
				roi_sizes.put(names.get(i), new MguiInteger(0));
				}
			for (int i = 0; i < rois.size(); i++){
				int roi = (int)rois.get(i).getValue();
				String name = roi_name_map.get(roi);
				if (name != null)
					roi_sizes.get(name).add(1);
				}
			}
		
		// Levels by ROIs
		HashMap<String,HashMap<String,SurfaceIntersectionStats>> roi_levels_frequencies = null;
		if (levels_map != null && roi_name_map != null){
			roi_levels_frequencies = new HashMap<String,HashMap<String,SurfaceIntersectionStats>>();
			ArrayList<String> roi_names = new ArrayList<String>(roi_name_map.getNames());
			ArrayList<String> level_names = new ArrayList<String>(levels_frequencies.keySet());
			for (int i = 0; i < level_names.size(); i++){
				HashMap<String,SurfaceIntersectionStats> map = new HashMap<String,SurfaceIntersectionStats>();
				roi_levels_frequencies.put(level_names.get(i), map);
				for (int j = 0; j < roi_names.size(); j++){
					map.put(roi_names.get(j), new SurfaceIntersectionStats());
					}
				}
			}
		
		// List of subjects
		String[] subjects = civet_dir.list(IoFunctions.getDirFilter());
		Arrays.sort(subjects);
		
		_param = parameters.get("subject_list");
		if (_param != null){
			File subject_list = new File(_param);
			if (!subject_list.exists()){
				log("surface_intersections_report: Subject list file '" + subject_list.getAbsolutePath() + "' not found.", 
					LoggingType.Errors);
				return false;
				}
			
			TreeSet<String> _subjects = new TreeSet<String>();
			try{
				BufferedReader reader = new BufferedReader(new FileReader(subject_list));
				String line = reader.readLine();
				while (line != null){
					_subjects.add(line);
					line = reader.readLine();
					}
			}catch (IOException e){
				log("surface_intersections_report: Error reading subject list: " + e.getMessage(), 
						LoggingType.Errors);
					return false;
				}
			
			if (subjects.length != _subjects.size()){
			
				ArrayList<String> new_subjects = new ArrayList<String>();
				for (int i = 0; i < subjects.length; i++){
					if (_subjects.contains(subjects[i]))
						new_subjects.add(subjects[i]);
					}
				
				subjects = new String[new_subjects.size()];
				for (int i = 0; i < new_subjects.size(); i++)
					subjects[i] = new_subjects.get(i);
				}
			}
		
		if (subjects.length == 0){
			log("surface_intersections_report: No subjects to process!", LoggingType.Errors);
			return false;
			}
		
		int success_count = 0, fail_count = 0;
		
		// Summary stats
		SurfaceIntersectionStats summary_stats = new SurfaceIntersectionStats();
		
		// Subject-wise stats
		//HashMap<String, SurfaceIntersectionStats> subject_stats = new HashMap<String, SurfaceIntersectionStats>();
		
		// Vertex-wise frequencies
		ArrayList<MguiNumber> frequencies_self_1 = null; // new ArrayList<MguiNumber>();
		ArrayList<MguiNumber> frequencies_self_2 = null; // new ArrayList<MguiNumber>();
		if (!no_self){
			frequencies_self_1 = new ArrayList<MguiNumber>();
			frequencies_self_2 = new ArrayList<MguiNumber>();
			}
		ArrayList<MguiNumber> frequencies_surf_1 = null; // new ArrayList<MguiNumber>();
		ArrayList<MguiNumber> frequencies_surf_2 = null; // new ArrayList<MguiNumber>();
		if (!no_surf){
			frequencies_surf_1 = new ArrayList<MguiNumber>();
			frequencies_surf_2 = new ArrayList<MguiNumber>();
			}
		
		HashMap<String,ArrayList<MguiNumber>> frequencies_self_1_levels = null;
		HashMap<String,ArrayList<MguiNumber>> frequencies_self_2_levels = null;
		HashMap<String,ArrayList<MguiNumber>> frequencies_surf_1_levels = null;
		HashMap<String,ArrayList<MguiNumber>> frequencies_surf_2_levels = null;
		
		// Levels / vertices
		if (levels_frequencies != null){
			if (!no_self){
				frequencies_self_1_levels = new HashMap<String,ArrayList<MguiNumber>>();
				frequencies_self_2_levels = new HashMap<String,ArrayList<MguiNumber>>();
				}
			if (!no_surf){
				frequencies_surf_1_levels = new HashMap<String,ArrayList<MguiNumber>>();
				frequencies_surf_2_levels = new HashMap<String,ArrayList<MguiNumber>>();
				}
			
			for (int i = 0; i < levels.size(); i++){
				String level = levels.get(i);
				if (!no_self){
					frequencies_self_1_levels.put(level, new ArrayList<MguiNumber>());
					frequencies_self_2_levels.put(level, new ArrayList<MguiNumber>());
					}
				if (!no_surf){
					frequencies_surf_1_levels.put(level, new ArrayList<MguiNumber>());
					frequencies_surf_2_levels.put(level, new ArrayList<MguiNumber>());
					}
				}
			
			}
		
		//ROI-wise stats
		HashMap<String,SurfaceIntersectionStats> roi_stats = null;
		if (rois != null){
			roi_stats = new HashMap<String,SurfaceIntersectionStats>();
			for (int i = 0; i < rois.size(); i++)
				roi_stats.put(roi_name_map.get(i), new SurfaceIntersectionStats());
			}
		
		try{
			BufferedWriter subject_report_writer = null;
			
			if (subject_report_file != null){
				subject_report_writer = new BufferedWriter(new FileWriter(subject_report_file));
				//Subject header
				String header = "subject_id";
				if (levels_map != null)
					header = header + "\tlevel";
				if (!no_self){
					header = header + "\tself_" + name_1 + "_faces_total\tself_" + name_1 + "_vertices_total";
					header = header + "\tself_" + name_2 + "_faces_total\tself_" + name_2 + "_vertices_total";
					}
				if (!no_surf){
					header = header + "\tsurf_" + name_1 + "_faces_total\tsurf_" + name_1 + "_vertices_total";
					header = header + "\tsurf_" + name_2 + "_faces_total\tsurf_" + name_2 + "_vertices_total";
					}
				if (roi_name_map != null){
					ArrayList<String> names = new ArrayList<String> (roi_name_map.getNames());
					Collections.sort(names);
					for (int i = 0; i < names.size(); i++){
						String roi = names.get(i);
						if (!no_self){
							header = header + "\tself_" + name_1 + "_faces_" + roi + "\tself_" + name_1 + "_vertices_" + roi;
							header = header + "\tself_" + name_2 + "_faces_" + roi + "\tself_" + name_2 + "_vertices_" + roi;
							}
						if (!no_surf){
							header = header + "\tsurf_" + name_1 + "_faces_" + roi + "\tsurf_" + name_1 + "_vertices_" + roi;
							header = header + "\tsurf_" + name_2 + "_faces_" + roi + "\tsurf_" + name_2 + "_vertices_" + roi;
							}
						}
					}
				subject_report_writer.write(header);
				}
			
			boolean init_frequencies = false;
		
			// For each subject, do self- and surface-surface comparison
			for (int i = 0; i < subjects.length; i++){
				String subject = subjects[i];
				String level = null;
				if (levels_map != null)
					level = levels_map.get(subject);
				
				File surface_dir = new File (civet_dir.getAbsolutePath() + File.separator + subject + File.separator + "surfaces");
				if (surface_dir.exists()){
					String[] surfaces = surface_dir.list(getObjFilter());
					File surf_1 = null, surf_2 = null;
					
					for (int j = 0; j < surfaces.length; j++){
						if (surfaces[j].endsWith(surface_pattern_1))
							surf_1 = new File(surface_dir.getAbsolutePath() + File.separator + surfaces[j]);
						if (surfaces[j].endsWith(surface_pattern_2))
							surf_2 = new File(surface_dir.getAbsolutePath() + File.separator + surfaces[j]);
						}
					
					if (surf_1 == null || surf_2 == null){
						log("surface_intersections_report: Both surfaces not found for subject '" + subject + "'.");
						fail_count++;
					}else{
						
						// Do comparison
						MincSurfaceLoader loader = new MincSurfaceLoader(surf_1);
						Mesh3DInt mesh_int_1 =  loader.loadSurface();
						Mesh3D mesh_1 = mesh_int_1.getMesh();
						loader.setFile(surf_2);
						Mesh3DInt mesh_int_2 =  loader.loadSurface();
						Mesh3D mesh_2 = mesh_int_2.getMesh();
						
						ArrayList<Integer> self_faces_1 = new ArrayList<Integer>(), self_faces_2 = new ArrayList<Integer>();
						ArrayList<Integer> self_vertices_1 = new ArrayList<Integer>(), self_vertices_2 = new ArrayList<Integer>();
						
						// Self-intersection
						if (!no_self){
							self_faces_1 = MeshFunctions.getSelfIntersections(mesh_1, search_max);
							self_faces_2 = MeshFunctions.getSelfIntersections(mesh_2, search_max);
							TreeSet<Integer> set = new TreeSet<Integer>();
							for (int j = 0; j < self_faces_1.size(); j++){
								MeshFace3D face = mesh_1.getFace(self_faces_1.get(j));
								set.add(face.A);
								set.add(face.B);
								set.add(face.C);
								}
							self_vertices_1 = new ArrayList<Integer>(set);
							set = new TreeSet<Integer>();
							for (int j = 0; j < self_faces_2.size(); j++){
								MeshFace3D face = mesh_1.getFace(self_faces_2.get(j));
								set.add(face.A);
								set.add(face.B);
								set.add(face.C);
								}
							self_vertices_2 = new ArrayList<Integer>(set);
							}
						
						ArrayList<Integer> surf_vertices_1 = new ArrayList<Integer>(), surf_vertices_2 = new ArrayList<Integer>(); 
						ArrayList<Integer> surf_faces_1 = new ArrayList<Integer>(), surf_faces_2 = new ArrayList<Integer>(); 
						
						// Surface-surface intersection
						if (!no_surf){
							ArrayList<ArrayList<Integer>> surf_ints = MeshFunctions.getIntersectingFaces(mesh_int_1.getMesh(), mesh_int_2.getMesh(), search_max);
							
							surf_faces_1 = surf_ints.get(0);
							TreeSet<Integer> set = new TreeSet<Integer>();
							for (int j = 0; j < surf_faces_1.size(); j++){
								MeshFace3D face = mesh_1.getFace(surf_faces_1.get(j));
								set.add(face.A);
								set.add(face.B);
								set.add(face.C);
								}
							surf_vertices_1 = new ArrayList<Integer>(set);
							
							surf_faces_2 = surf_ints.get(1);
							set = new TreeSet<Integer>();
							for (int j = 0; j < surf_faces_2.size(); j++){
								MeshFace3D face = mesh_1.getFace(surf_faces_2.get(j));
								set.add(face.A);
								set.add(face.B);
								set.add(face.C);
								}
							surf_vertices_2 = new ArrayList<Integer>(set);
							}
						
						// Summary stats
						summary_stats.self_faces_1 += self_faces_1.size();
						summary_stats.self_faces_2 += self_faces_2.size();
						summary_stats.self_vertices_1 += self_vertices_1.size();
						summary_stats.self_vertices_2 += self_vertices_2.size();
						summary_stats.surf_faces_1 += surf_faces_1.size();
						summary_stats.surf_faces_2 += surf_faces_2.size();
						summary_stats.surf_vertices_1 += surf_vertices_1.size();
						summary_stats.surf_vertices_2 += surf_vertices_2.size();
						
						// ROI-wise stats
						HashMap<String,SurfaceIntersectionStats> roi_subject = null;
						if (rois != null){
							roi_subject = new HashMap<String,SurfaceIntersectionStats>();
							ArrayList<String> names = new ArrayList<String>(roi_name_map.getNames());
							Collections.sort(names);
							for (int j = 0; j < names.size(); j++){
								String roi = names.get(j);
								SurfaceIntersectionStats stats = new SurfaceIntersectionStats();
								SurfaceIntersectionStats total_stats = roi_stats.get(roi);
								roi_subject.put(roi, stats);
								
								for (int k = 0; k < self_vertices_1.size(); k++){
									MguiInteger index = (MguiInteger)rois.get(self_vertices_1.get(k));
									if (index != null){
										String _roi = roi_name_map.get(index.getInt());
										if (_roi != null && _roi.equals(roi)){
											stats.self_vertices_1++;
											total_stats.self_vertices_1++;
											}
										}
									}
								
								for (int k = 0; k < self_vertices_2.size(); k++){
									MguiInteger index = (MguiInteger)rois.get(self_vertices_2.get(k));
									if (index != null){
										String _roi = roi_name_map.get(index.getInt());
										if (_roi != null && _roi.equals(roi)){
											stats.self_vertices_2++;
											total_stats.self_vertices_2++;
											}
										}
									}
								
								for (int k = 0; k < surf_vertices_1.size(); k++){
									MguiInteger index = (MguiInteger)rois.get(surf_vertices_1.get(k));
									if (index != null){
										String _roi = roi_name_map.get(index.getInt());
										if (_roi != null && _roi.equals(roi)){
											stats.surf_vertices_1++;
											total_stats.surf_vertices_1++;
											}
										}
									}
								
								for (int k = 0; k < surf_vertices_2.size(); k++){
									MguiInteger index = (MguiInteger)rois.get(surf_vertices_2.get(k));
									if (index != null){
										String _roi = roi_name_map.get(index.getInt());
										if (_roi != null && _roi.equals(roi)){
											stats.surf_vertices_2++;
											total_stats.surf_vertices_2++;
											}
										}
									}
								
								for (int k = 0; k < self_faces_1.size(); k++){
									MeshFace3D face = mesh_1.getFace(self_faces_1.get(k));
									boolean found = false;
									for (int l = 0; !found && l < 3; l++){
										MguiInteger index = (MguiInteger)rois.get(face.getNode(l));
										if (index != null){
											String _roi = roi_name_map.get(index.getInt());
											found = (_roi != null && _roi.equals(roi));
											}
										}
									if (found){
										stats.self_faces_1++;
										total_stats.self_faces_1++;
										}
									}
								
								for (int k = 0; k < self_faces_2.size(); k++){
									MeshFace3D face = mesh_2.getFace(self_faces_2.get(k));
									boolean found = false;
									for (int l = 0; !found && l < 3; l++){
										MguiInteger index = (MguiInteger)rois.get(face.getNode(l));
										if (index != null){
											String _roi = roi_name_map.get(index.getInt());
											found = (_roi != null && _roi.equals(roi));
											}
										}
									if (found){
										stats.self_faces_2++;
										total_stats.self_faces_2++;
										}
									}
								
								for (int k = 0; k < surf_faces_1.size(); k++){
									MeshFace3D face = mesh_1.getFace(surf_faces_1.get(k));
									boolean found = false;
									for (int l = 0; !found && l < 3; l++){
										MguiInteger index = (MguiInteger)rois.get(face.getNode(l));
										if (index != null){
											String _roi = roi_name_map.get(index.getInt());
											found = (_roi != null && _roi.equals(roi));
											}
										}
									if (found){
										stats.surf_faces_1++;
										total_stats.surf_faces_1++;
										}
									}
								
								for (int k = 0; k < self_faces_2.size(); k++){
									MeshFace3D face = mesh_2.getFace(surf_faces_2.get(k));
									boolean found = false;
									for (int l = 0; !found && l < 3; l++){
										MguiInteger index = (MguiInteger)rois.get(face.getNode(l));
										if (index != null){
											String _roi = roi_name_map.get(index.getInt());
											found = (_roi != null && _roi.equals(roi));
											}
										}
									if (found){
										stats.surf_faces_2++;
										total_stats.surf_faces_2++;
										}
									}
								
								}
							}
						
						// Subject-wise stats to file
						if (subject_report_file != null){
							String line = "\n" + subject + "\t";
							if (levels_map != null)
								line = line + levels_map.get(subject) + "\t";
							if (!no_self){
								line = line + self_faces_1.size() + "\t" + self_vertices_1.size() + "\t";
								line = line + self_faces_2.size() + "\t" + self_vertices_2.size() + "\t";
								}
							if (!no_surf){
								line = line + surf_faces_1.size() + "\t" + surf_vertices_1.size() + "\t";
								line = line + surf_faces_2.size() + "\t" + surf_vertices_2.size();
								}
							if (rois != null){
								ArrayList<String> names = new ArrayList<String> (roi_name_map.getNames());
								Collections.sort(names);
								for (int j = 0; j < names.size(); j++){
									String roi = names.get(j);
									double pct_size = 1.0; // roi_sizes.get(roi).getValue() / 100; 
									SurfaceIntersectionStats stats = roi_subject.get(roi);
									if (!no_self){
										line = line + "\t" + stats.self_faces_1 / pct_size + "\t" + stats.self_vertices_1 / pct_size;
										line = line + "\t" + stats.self_faces_2 / pct_size + "\t" + stats.self_vertices_2 / pct_size;
										}
									if (!no_surf){
										line = line + "\t" + stats.surf_faces_1 / pct_size + "\t" + stats.surf_vertices_1 / pct_size;
										line = line + "\t" + stats.surf_faces_2 / pct_size+ "\t" + stats.surf_vertices_2 / pct_size;
										}
									}
								}
							subject_report_writer.write(line);
							}
						
						// Vertex-wise frequencies
						if (frequencies_self_1 != null || frequencies_surf_1 != null){
							
							ArrayList<MguiNumber> intersections_self_1 = null;
							ArrayList<MguiNumber> intersections_self_2 = null;
							ArrayList<MguiNumber> intersections_surf_1 = null;
							ArrayList<MguiNumber> intersections_surf_2 = null;
							
							if (subject_intersection_file_pattern != null){
								if (!no_self){
									intersections_self_1 = new ArrayList<MguiNumber>();
									intersections_self_2 = new ArrayList<MguiNumber>();
									}
								if (!no_surf){
									intersections_surf_1 = new ArrayList<MguiNumber>();
									intersections_surf_2 = new ArrayList<MguiNumber>();
									}
								
								for (int j = 0; j < mesh_1.getSize(); j++){
									if (!no_self) intersections_self_1.add(new MguiInteger(0));
									if (!no_surf) intersections_surf_1.add(new MguiInteger(0));
									}
								
								for (int j = 0; j < mesh_1.getSize(); j++){
									if (!no_self) intersections_self_2.add(new MguiInteger(0));
									if (!no_surf) intersections_surf_2.add(new MguiInteger(0));
									}
								}
							
							if (!init_frequencies){
								init_frequencies = true;
								// Init arrays on first iteration
								for (int j = 0; j < mesh_1.getSize(); j++){
									if (!no_self) frequencies_self_1.add(new MguiInteger(0));
									if (!no_surf) frequencies_surf_1.add(new MguiInteger(0));
									
									if (levels != null){
										for (int k = 0; k < levels.size(); k++){
											String _level = levels.get(k);
											if (!no_self) frequencies_self_1_levels.get(_level).add(new MguiInteger(0));
											if (!no_surf) frequencies_surf_1_levels.get(_level).add(new MguiInteger(0));
											}
										}
									}
								for (int j = 0; j < mesh_2.getSize(); j++){
									if (!no_self) frequencies_self_2.add(new MguiInteger(0));
									if (!no_surf) frequencies_surf_2.add(new MguiInteger(0));
									
									if (levels != null){
										for (int k = 0; k < levels.size(); k++){
											String _level = levels.get(k);
											if (!no_self) frequencies_self_2_levels.get(_level).add(new MguiInteger(0));
											if (!no_surf) frequencies_surf_2_levels.get(_level).add(new MguiInteger(0));
											}
										}
									}
								}
							
							if (!no_self){
								for (int j = 0; j < self_vertices_1.size(); j++){
									frequencies_self_1.get(self_vertices_1.get(j)).add(1);
									if (level != null)
										frequencies_self_1_levels.get(level).get(self_vertices_1.get(j)).add(1);
									if (intersections_self_1 != null)
										intersections_self_1.get(self_vertices_1.get(j)).add(1);
									}
								}
							
							boolean dir_exists = false, dir_failed = false;
							if (intersections_self_1 != null){
								File file = new File(civet_dir.getAbsolutePath() + 
													 File.separator + subject + 
													 File.separator + "intersections");
								if (!file.exists() && !file.mkdir()){
									log("surface_intersections_report: Could not create intersections folder");
									dir_failed = true;
								}else{
									dir_exists = true;
									file = new File(file.getAbsolutePath() + File.separator + 
													subject_intersection_file_pattern + "_self_" + name_1 + ".txt");
									MincSurfaceDataWriter surface_data_writer = new MincSurfaceDataWriter(file);
									if (!surface_data_writer.writeValues(intersections_self_1)){
										log("surface_intersections_report: Could not write intersections to '" + 
											file.getAbsolutePath() + "'.");
										}
									}
								}
							
							if (!no_self){
								for (int j = 0; j < self_vertices_2.size(); j++){
									frequencies_self_2.get(self_vertices_2.get(j)).add(1);
									if (level != null)
										frequencies_self_2_levels.get(level).get(self_vertices_2.get(j)).add(1);
									if (intersections_self_2 != null)
										intersections_self_2.get(self_vertices_2.get(j)).add(1);
									}
								}
							
							if (!dir_failed && intersections_self_2 != null){
								File file = new File(civet_dir.getAbsolutePath() + 
													 File.separator + subject + 
													 File.separator + "intersections" + 
													 File.separator + subject_intersection_file_pattern + 
													 "_self_" + name_2 + ".txt");
								MincSurfaceDataWriter surface_data_writer = new MincSurfaceDataWriter(file);
								if (!surface_data_writer.writeValues(intersections_self_2)){
									log("surface_intersections_report: Could not write intersections to '" + 
										file.getAbsolutePath() + "'.");
									}
								}
							
							if (!no_surf){
								for (int j = 0; j < surf_vertices_1.size(); j++){
									frequencies_surf_1.get(surf_vertices_1.get(j)).add(1);
									if (level != null)
										frequencies_surf_1_levels.get(level).get(surf_vertices_1.get(j)).add(1);
									if (intersections_surf_1 != null)
										intersections_surf_1.get(surf_vertices_1.get(j)).add(1);
									}
								}
							
							if (!dir_failed && intersections_surf_1 != null){
								File file = new File(civet_dir.getAbsolutePath() + 
										 File.separator + subject + 
										 File.separator + "intersections");
								if (!dir_exists){
									if (!file.exists() && !file.mkdir()){
										log("surface_intersections_report: Could not create intersections folder");
										dir_failed = true;
										}
									}
								if (!dir_failed){
									dir_exists = true;
									file = new File(file.getAbsolutePath() + 
													File.separator + subject_intersection_file_pattern + 
													"_surf_" + name_1 + ".txt");
									MincSurfaceDataWriter surface_data_writer = new MincSurfaceDataWriter(file);
									if (!surface_data_writer.writeValues(intersections_surf_1)){
										log("surface_intersections_report: Could not write intersections to '" + 
											file.getAbsolutePath() + "'.");
										}
									}
								}
							
							if (!no_surf){
								for (int j = 0; j < surf_vertices_2.size(); j++){
									frequencies_surf_2.get(surf_vertices_2.get(j)).add(1);
									if (level != null)
										frequencies_surf_2_levels.get(level).get(surf_vertices_2.get(j)).add(1);
									if (intersections_surf_2 != null)
										intersections_surf_2.get(surf_vertices_2.get(j)).add(1);
									}
								}
							
							if (!dir_failed && intersections_surf_2 != null){
								File file = new File(civet_dir.getAbsolutePath() + 
													 File.separator + subject + 
													 File.separator + "intersections" + 
													 File.separator + subject_intersection_file_pattern + 
													 "_surf_" + name_2 + ".txt");
								MincSurfaceDataWriter surface_data_writer = new MincSurfaceDataWriter(file);
								if (!surface_data_writer.writeValues(intersections_surf_2)){
									log("surface_intersections_report: Could not write intersections to '" + 
										file.getAbsolutePath() + "'.");
									}
								}
							}
						
						// Levels stats
						if (levels_map != null){
							level_sizes.get(level).add(1);
							if (level != null){
								SurfaceIntersectionStats stats = levels_frequencies.get(level);
								stats.self_faces_1 += self_faces_1.size();
								stats.self_faces_2 += self_faces_2.size();
								stats.self_vertices_1 += self_vertices_1.size();
								stats.self_vertices_2 += self_vertices_2.size();
								stats.surf_faces_1 += surf_faces_1.size();
								stats.surf_faces_2 += surf_faces_2.size();
								stats.surf_vertices_1 += surf_vertices_1.size();
								stats.surf_vertices_2 += surf_vertices_2.size();
								
								// ROIs?
								if (roi_levels_frequencies != null){
									HashMap<String,SurfaceIntersectionStats> r_stats = roi_levels_frequencies.get(level);
									
									ArrayList<String> names = new ArrayList<String> (roi_name_map.getNames());
									Collections.sort(names);
									for (int j = 0; j < names.size(); j++){
										String roi = names.get(j);
										stats = r_stats.get(roi);
										SurfaceIntersectionStats s_stats = roi_subject.get(roi);
										
										stats.self_faces_1 += s_stats.self_faces_1;
										stats.self_faces_2 += s_stats.self_faces_2;
										stats.self_vertices_1 += s_stats.self_vertices_1;
										stats.self_vertices_2 += s_stats.self_vertices_2;
										stats.surf_faces_1 += s_stats.surf_faces_1;
										stats.surf_faces_2 += s_stats.surf_faces_2;
										stats.surf_vertices_1 += s_stats.surf_vertices_1;
										stats.surf_vertices_2 += s_stats.surf_vertices_2;
										}
									}
								}
							}
						log("surface_intersections_report: Processed subject '" + subject + "'.");
						success_count++;
						}
				}else{
					log("surface_intersections_report: Invalid subject dir '" + subject + "'.");
					fail_count++;
					}
				}
			
			if (subject_report_file != null){
				subject_report_writer.close();
				log("surface_intersections_report: Wrote subject table report to '" + subject_report_file.getAbsolutePath() + "'.");
				}
			
			// Write summary report
			BufferedWriter report_writer = new BufferedWriter(new FileWriter(report_file));
			
			double n = success_count; // subjects.length; 
			
			String line = "Level\tN";
			if (!no_self){
				line = line + "\tself_" + name_1 + "_faces\tself_" + name_1 + "_vertices";
				line = line + "\tself_" + name_2 + "_faces\tself_" + name_2 + "_vertices";
				}
			if (!no_surf){
				line = line + "\tsurf_" + name_1 + "_faces\tsurf_" + name_1 + "_vertices";
				line = line + "\tsurf_" + name_2 + "_faces\tsurf_" + name_2 + "_vertices";
				}
			report_writer.write(line);
			
			line = "\nOverall\t" + (int)n;
			if (!no_self){
				line = line + "\t" + (double)summary_stats.self_faces_1 / n + "\t" + (double)summary_stats.self_vertices_1 / n;
				line = line + "\t" + (double)summary_stats.self_faces_2 / n + "\t" + (double)summary_stats.self_vertices_2 / n;
				}
			if (!no_surf){
				line = line + "\t" + (double)summary_stats.surf_faces_1 / n + "\t" + (double)summary_stats.surf_vertices_1 / n;
				line = line + "\t" + (double)summary_stats.surf_faces_2 / n + "\t" + (double)summary_stats.surf_vertices_2 / n;
				}
			report_writer.write(line);
			
			//Levels?
			if (levels_map != null){
				
				
				for (int i = 0; i < levels.size(); i++){
					String level = levels.get(i);
					double size = level_sizes.get(level).getValue();
					SurfaceIntersectionStats stats = levels_frequencies.get(level);
					line = "\n" + level + "\t" + (int)size;
					if (!no_self){
						line = line + "\t" + (double)stats.self_faces_1 / size + "\t" + (double)stats.self_vertices_1 / size;
						line = line + "\t" + (double)stats.self_faces_2 / size + "\t" + (double)stats.self_vertices_2 / size;
						}
					if (!no_surf){
						line = line + "\t" + (double)stats.surf_faces_1 / size + "\t" + (double)stats.surf_vertices_1 / size;
						line = line + "\t" + (double)stats.surf_faces_2 / size + "\t" + (double)stats.surf_vertices_2 / size;
						}
					report_writer.write(line);
					}
				
				}
			
			report_writer.close();
			log("surface_intersections_report: Wrote summary report to '" + report_file.getAbsolutePath() + "'.");
			
			MincSurfaceDataWriter surface_data_writer;
			
			// Write vertex-wise frequencies
			if (frequency_file_self_1 != null){
				surface_data_writer = new MincSurfaceDataWriter(frequency_file_self_1);
				for (int i = 0; i < frequencies_self_1.size(); i++)
					frequencies_self_1.get(i).divide(n).multiply(100);
				if (!surface_data_writer.writeValues(frequencies_self_1))
					log("surface_intersections_report: Could not write to file '" + frequency_file_self_1.getAbsolutePath() + "'.",
							LoggingType.Errors);
				surface_data_writer = new MincSurfaceDataWriter(frequency_file_self_2);
				for (int i = 0; i < frequencies_self_2.size(); i++)
					frequencies_self_2.get(i).divide(n).multiply(100);
				if (!surface_data_writer.writeValues(frequencies_self_2))
					log("surface_intersections_report: Could not write to file '" + frequency_file_self_2.getAbsolutePath() + "'.",
							LoggingType.Errors);
				log("surface_intersections_report: Wrote vertex-wise self-intersection frequency maps.");
				}
			
			if (frequency_file_surf_1 != null){
				surface_data_writer = new MincSurfaceDataWriter(frequency_file_surf_1);
				for (int i = 0; i < frequencies_surf_1.size(); i++)
					frequencies_surf_1.get(i).divide(n).multiply(100);
				if (!surface_data_writer.writeValues(frequencies_surf_1))
					log("surface_intersections_report: Could not write to file '" + frequency_file_surf_1.getAbsolutePath() + "'.",
							LoggingType.Errors);
				surface_data_writer = new MincSurfaceDataWriter(frequency_file_surf_2);
				for (int i = 0; i < frequencies_surf_2.size(); i++)
					frequencies_surf_2.get(i).divide(n).multiply(100);
				if (!surface_data_writer.writeValues(frequencies_surf_2))
					log("surface_intersections_report: Could not write to file '" + frequency_file_surf_2.getAbsolutePath() + "'.",
							LoggingType.Errors);
				log("surface_intersections_report: Wrote vertex-wise surface-surface frequency maps.");
				}
			
			if (frequency_files_levels != null){
				
				if (!no_self){
					HashMap<String,File> level_files = frequency_files_levels.get(frequency_file_self_1);
					for (int i = 0; i < levels.size(); i++){
						String level = levels.get(i);
						File file = level_files.get(i);
						surface_data_writer = new MincSurfaceDataWriter(file);
						int size = level_sizes.get(level).getInt();
						ArrayList<MguiNumber> list = frequencies_self_1_levels.get(level);
						for (int j = 0; j < list.size(); j++)
							list.get(j).divide(size).multiply(100);
						if (!surface_data_writer.writeValues(list))
							log("surface_intersections_report: Could not write to file '" + file.getAbsolutePath() + "'.",
									LoggingType.Errors);
						}
					level_files = frequency_files_levels.get(frequency_file_self_2);
					for (int i = 0; i < levels.size(); i++){
						String level = levels.get(i);
						File file = level_files.get(i);
						surface_data_writer = new MincSurfaceDataWriter(file);
						ArrayList<MguiNumber> list = frequencies_self_2_levels.get(level);
						int size = level_sizes.get(level).getInt();
						for (int j = 0; j < list.size(); j++)
							list.get(j).divide(size).multiply(100);
						if (!surface_data_writer.writeValues(list))
							log("surface_intersections_report: Could not write to file '" + file.getAbsolutePath() + "'.",
									LoggingType.Errors);
						}
					}
				
				if (!no_surf){
					HashMap<String,File> level_files = frequency_files_levels.get(frequency_file_surf_1);
					for (int i = 0; i < levels.size(); i++){
						String level = levels.get(i);
						File file = level_files.get(level);
						surface_data_writer = new MincSurfaceDataWriter(file);
						ArrayList<MguiNumber> list = frequencies_surf_1_levels.get(level);
						if (!surface_data_writer.writeValues(list))
							log("surface_intersections_report: Could not write to file '" + file.getAbsolutePath() + "'.",
									LoggingType.Errors);
						}
					level_files = frequency_files_levels.get(frequency_file_surf_2);
					for (int i = 0; i < levels.size(); i++){
						String level = levels.get(i);
						File file = level_files.get(level);
						surface_data_writer = new MincSurfaceDataWriter(file);
						ArrayList<MguiNumber> list = frequencies_surf_2_levels.get(level);
						if (!surface_data_writer.writeValues(list))
							log("surface_intersections_report: Could not write to file '" + file.getAbsolutePath() + "'.",
									LoggingType.Errors);
						}
					}
				
				}
			
			// Write ROI table
			if (roi_report_file != null){
				BufferedWriter writer = new BufferedWriter(new FileWriter(roi_report_file));
				//Header
				line = "Level\tROI";
				if (!no_self){
					line = line + "\tself_" + name_1 + "_faces\tself_" + name_1 + "_vertices";
					line = line + "\tself_" + name_2 + "_faces\tself_" + name_2 + "_vertices";
					}
				if (!no_surf){
					line = line + "\tsurf_" + name_1 + "_faces\tsurf_" + name_1 + "_vertices";
					line = line + "\tsurf_" + name_2 + "_faces\tsurf_" + name_2 + "_vertices";
					}
				writer.write(line);
				
				ArrayList<String> roi_names = new ArrayList<String>(roi_name_map.getNames());
				Collections.sort(roi_names);
				String level = "Overall";
				
				for (int i = 0; i < roi_names.size(); i++){
					String roi = roi_names.get(i);
					double pct_size = roi_sizes.get(roi).getValue() * n / 100.0;
					SurfaceIntersectionStats stats = roi_stats.get(roi);
					
					line = "\n" + level + "\t" + roi;
					if (!no_self){
						line = line + "\t" + (double)stats.self_faces_1 / pct_size + "\t" + (double)stats.self_vertices_1 / pct_size;
						line = line + "\t" + (double)stats.self_faces_2 / pct_size + "\t" + (double)stats.self_vertices_2 / pct_size;
						}
					if (!no_surf){
						line = line + "\t" + (double)stats.surf_faces_1 / pct_size + "\t" + (double)stats.surf_vertices_1 / pct_size;
						line = line + "\t" + (double)stats.surf_faces_2 / pct_size + "\t" + (double)stats.surf_vertices_2 / pct_size;
						}
					writer.write(line);
					}
				
				
				//Levels/ROIs
				if (levels != null){
					for (int i = 0; i < levels.size(); i++){
						level = levels.get(i);
						HashMap<String,SurfaceIntersectionStats> map = roi_levels_frequencies.get(level);
						for (int j = 0; j < roi_names.size(); j++){
							String roi = roi_names.get(j);
							double pct_size = roi_sizes.get(roi).getValue() * level_sizes.get(level).getValue() / 100.0;
							SurfaceIntersectionStats stats = map.get(roi);
							
							line = "\n" + level + "\t" + roi;
							if (!no_self){
								line = line + "\t" + (double)stats.self_faces_1 / pct_size + "\t" + (double)stats.self_vertices_1 / pct_size;
								line = line + "\t" + (double)stats.self_faces_2 / pct_size + "\t" + (double)stats.self_vertices_2 / pct_size;
								}
							if (!no_surf){
								line = line + "\t" + (double)stats.surf_faces_1 / pct_size + "\t" + (double)stats.surf_vertices_1 / pct_size;
								line = line + "\t" + (double)stats.surf_faces_2 / pct_size + "\t" + (double)stats.surf_vertices_2 / pct_size;
								}
							writer.write(line);
							}
						
						}
					
					}
				
				writer.close();
				log("surface_intersections_report: Wrote ROI report to '" + roi_report_file.getAbsolutePath() + "'.");
				}
		
		}catch (IOException e){
			log("surface_intersections_report: IO exception encountered: " + e.getMessage(), LoggingType.Errors);
			return false;
			}
		
		return true;
	}
	
	/*****************************************************************
	 * Generates surface intersection reports from vertex-wise intersection data contained in subject
	 * directories (i.e., as created by {@code surface_intersections_report}). Reports are the same as that method,
	 * except they obviously do not include information about faces.
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>{@code civet_dir} - The Civet output directory
	 * <li>{@code report_file} - File name for the summary report
	 * <li>{@code intersection_file_pattern} - Input file name pattern identifying the vertex-wise intersections for each
	 * 										   subject (0 = no intersection; 1 = intersection); ends with
	 * <li>{@code subject_report_file} [Optional] - Output file for subject-wise table report
	 * <li>{@code frequency_file_pattern} [Optional] - Output file name pattern for the total vertex-wise frequencies. Will create two
	 * 												   file: one with absolute frequency, the other for percentage of group;
	 * 												   these will append "_abs.txt" and "_pct.txt" respectively.
	 * 
	 * <li>{@code levels_file} [Optional] - Tab-delimited text file specifying a set of levels to break up the summary; 
	 * 										format is [subject_id]\t[level]
	 * <li>{@code roi_report_file} [Optional] - File for ROI-wise table report
	 * <li>{@code roi_file} [Optional] - Vertex-wise ROI assignments (text file, one integer per line)
	 * <li>{@code roi_name_map} [Optional] - Name map for ROIs
	 * <li>{@code subject_list} [Optional] - Operates only on subjects listed in a text file
	 * </ul>
	 * 
	 * @return
	 */
	boolean surface_intersections_report_from_files(){
	
		String _param = parameters.get("civet_dir");
		if (_param == null){
			log("surface_intersections_report_from_files: civet_dir is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File civet_dir = new File(_param);
		if (!civet_dir.exists() || !civet_dir.isDirectory()){
			log("surface_intersections_report_from_files: Cannot find civet_dir: " + civet_dir.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("report_file");
		if (_param == null){
			log("surface_intersections_report_from_files: report_file is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		
		File report_file = new File(_param);
		if (report_file.exists() && !report_file.delete()){
			log("surface_intersections_report_from_files: Cannot overwrite report_file: " + report_file.getAbsolutePath(),
					LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("intersection_file_pattern");
		if (_param == null){
			log("surface_intersections_report_from_files: intersection_file_pattern is a required parameter.",
					LoggingType.Errors);
			return false;
			}
		String intersection_file_pattern = _param;
		
		HashMap<String,String> levels_map = null;
		HashMap<String,MguiInteger> levels_frequencies = null;
		_param = parameters.get("levels_file");
		if (_param != null){
			File levels_file = new File(_param);
			if (!levels_file.exists()){
				log("surface_intersections_report_from_files: Cannot find levels_file: " + levels_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			levels_map = new HashMap<String,String>();
			levels_frequencies = new HashMap<String,MguiInteger>();
			try{
				BufferedReader reader = new BufferedReader(new FileReader(levels_file));
				String line = reader.readLine();
				while (line != null){
					String[] parts = line.split("\t");
					levels_map.put(parts[0], parts[1]);
					if (!levels_frequencies.containsKey(parts[1]))
						levels_frequencies.put(parts[1], new MguiInteger(0));
					line = reader.readLine();
					}
			}catch (IOException e){
				log("surface_intersections_report_from_files: Error reading levels_file: " + levels_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
		
		// Level sizes
		HashMap<String,MguiInteger> level_sizes = null;
		ArrayList<String> levels = null;
		if (levels_map != null){
			level_sizes = new HashMap<String,MguiInteger>();
			levels = new ArrayList<String>(levels_frequencies.keySet());
			Collections.sort(levels);
			for (int i = 0; i < levels.size(); i++){
				level_sizes.put(levels.get(i), new MguiInteger(0));
				}
			}
		
		_param = parameters.get("frequency_file_pattern");
		File frequency_file_sum = null, frequency_file_pct = null;
		
		if (_param != null){
			frequency_file_sum = new File(_param + "_sum.txt");
			if (frequency_file_sum.exists() && !frequency_file_sum.delete()){
				log("surface_intersections_report_from_files: Cannot overwrite frequency_file: " + frequency_file_sum.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			frequency_file_pct = new File(_param + "_pct.txt");
			if (frequency_file_pct.exists() && !frequency_file_pct.delete()){
				log("surface_intersections_report_from_files: Cannot overwrite frequency_file: " + frequency_file_pct.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
				
		HashMap<String,File> frequency_files_levels_sum = null;
		HashMap<String,File> frequency_files_levels_pct = null; 
		HashMap<String,ArrayList<MguiNumber>> frequencies_levels = null;
		ArrayList<MguiNumber> frequencies = null;
			
		//Vertex-wise level frequencies
		if (frequency_file_sum != null && levels != null){
			frequency_files_levels_sum = new HashMap<String, File>();
			frequency_files_levels_pct = new HashMap<String, File>();
			frequencies_levels = new HashMap<String,ArrayList<MguiNumber>>();
			frequencies = new ArrayList<MguiNumber>();
			
			for (int i = 0; i < levels.size(); i++){
				String level = levels.get(i);
				File frequency_file = new File(_param + "_sum_" + level + ".txt");
				if (frequency_file.exists() && !frequency_file.delete()){
					log("surface_intersections_report_from_files: Cannot overwrite frequency_file: " + frequency_file.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
				frequency_files_levels_sum.put(level, frequency_file);
				frequency_file = new File(_param + "_pct_" + level + ".txt");
				if (frequency_file.exists() && !frequency_file.delete()){
					log("surface_intersections_report_from_files: Cannot overwrite frequency_file: " + frequency_file.getAbsolutePath(),
							LoggingType.Errors);
					return false;
					}
				frequency_files_levels_pct.put(level, frequency_file);
				frequencies_levels.put(level, new ArrayList<MguiNumber>());
				}
			
			}
		 
				
		File subject_report_file = null;
		_param = parameters.get("subject_report_file");
		if (_param != null){
			subject_report_file = new File(_param);
			if (subject_report_file.exists() && !subject_report_file.delete()){
				log("surface_intersections_report_from_files: Cannot overwrite subject_report_file: " + subject_report_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
		
		File roi_report_file = null;
		_param = parameters.get("roi_report_file");
		if (_param != null){
			roi_report_file = new File(_param);
			if (roi_report_file.exists() && !roi_report_file.delete()){
				log("surface_intersections_report_from_files: Cannot overwrite roi_report_file: " + roi_report_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			}
		
		ArrayList<MguiNumber> rois = null;
		_param = parameters.get("roi_file");
		if (_param != null){
			File roi_file = new File(_param);
			if (!roi_file.exists()){
				log("surface_intersections_report_from_files: cannot find ROI file: " + roi_file + ".",
						LoggingType.Errors);
				return false;
				}
			MincSurfaceDataLoader loader = new MincSurfaceDataLoader(roi_file, MincSurfaceDataLoader.FORMAT_INT);
			rois = loader.loadValues();
			
			}
		
		NameMap roi_name_map = null;
		
		_param = parameters.get("roi_name_map");
		if (_param != null){
			File roi_name_map_file = new File(_param);
			if (!roi_name_map_file.exists()){
				log("surface_intersections_report_from_files: cannot find ROI name map file: " + roi_name_map_file + ".");
				return false;
				}
			NameMapLoader loader = new NameMapLoader(roi_name_map_file);
			
			roi_name_map = new NameMap();
			
			if (!loader.loadNameMap(roi_name_map)){
				log("surface_intersections_report_from_files: could not load ROI name map from file: " + roi_name_map_file + ".");
				return false;
				}
		}else if (rois != null){
			// List of ROIS is integers
			roi_name_map = new NameMap();
			
			for (int i = 0; i < rois.size(); i++){
				int roi = (int)rois.get(i).getValue();
				if (!roi_name_map.contains(roi))
					roi_name_map.add(roi, "" + roi);
				}
			}
		
		// ROI sizes
		HashMap<String,MguiInteger> roi_sizes = null;
		if (roi_name_map != null){
			roi_sizes = new HashMap<String,MguiInteger>();
			ArrayList<String> names = new ArrayList<String>(roi_name_map.getNames());
			for (int i = 0; i < names.size(); i++){
				roi_sizes.put(names.get(i), new MguiInteger(0));
				}
			for (int i = 0; i < rois.size(); i++){
				int roi = (int)rois.get(i).getValue();
				String name = roi_name_map.get(roi);
				if (name != null)
					roi_sizes.get(name).add(1);
				}
			}
		
		//ROI-wise stats
		HashMap<String,MguiInteger> roi_stats = null;
		if (rois != null){
			roi_stats = new HashMap<String,MguiInteger>();
			for (int i = 0; i < rois.size(); i++)
				roi_stats.put(roi_name_map.get(i), new MguiInteger(0));
			}
		
		// Levels by ROIs
		HashMap<String,HashMap<String,MguiInteger>> roi_levels_frequencies = null;
		if (levels_map != null && roi_name_map != null){
			roi_levels_frequencies = new HashMap<String,HashMap<String,MguiInteger>>();
			ArrayList<String> roi_names = new ArrayList<String>(roi_name_map.getNames());
			ArrayList<String> level_names = new ArrayList<String>(levels_frequencies.keySet());
			for (int i = 0; i < level_names.size(); i++){
				HashMap<String,MguiInteger> map = new HashMap<String,MguiInteger>();
				roi_levels_frequencies.put(level_names.get(i), map);
				for (int j = 0; j < roi_names.size(); j++){
					map.put(roi_names.get(j), new MguiInteger(0));
					}
				}
			}
		
		// List of subjects
		String[] subjects = civet_dir.list(IoFunctions.getDirFilter());
		Arrays.sort(subjects);
		
		_param = parameters.get("subject_list");
		if (_param != null){
			File subject_list = new File(_param);
			if (!subject_list.exists()){
				log("surface_intersections_report_from_files: Subject list file '" + subject_list.getAbsolutePath() + "' not found.", 
					LoggingType.Errors);
				return false;
				}
			
			TreeSet<String> _subjects = new TreeSet<String>();
			try{
				BufferedReader reader = new BufferedReader(new FileReader(subject_list));
				String line = reader.readLine();
				while (line != null){
					_subjects.add(line);
					line = reader.readLine();
					}
			}catch (IOException e){
				log("surface_intersections_report_from_files: Error reading subject list: " + e.getMessage(), 
						LoggingType.Errors);
					return false;
				}
			
			if (subjects.length != _subjects.size()){
			
				ArrayList<String> new_subjects = new ArrayList<String>();
				for (int i = 0; i < subjects.length; i++){
					if (_subjects.contains(subjects[i]))
						new_subjects.add(subjects[i]);
					}
				
				subjects = new String[new_subjects.size()];
				for (int i = 0; i < new_subjects.size(); i++)
					subjects[i] = new_subjects.get(i);
				}
			}
		
		if (subjects.length == 0){
			log("surface_intersections_report_from_files: No subjects to process!", LoggingType.Errors);
			return false;
			}
		
		int success_count = 0, fail_count = 0;
		
		// Summary stats
		int total_intersections = 0;
		
		
		
		try{
			BufferedWriter subject_report_writer = null;
			
			if (subject_report_file != null){
				subject_report_writer = new BufferedWriter(new FileWriter(subject_report_file));
				//Subject header
				String header = "subject_id";
				if (levels_map != null)
					header = header + "\tlevel";
				
				header = header + "\toverall_sum\toverall_pct";
					
				if (roi_name_map != null){
					ArrayList<String> names = new ArrayList<String> (roi_name_map.getNames());
					Collections.sort(names);
					for (int i = 0; i < names.size(); i++){
						String roi = names.get(i);
						header = header + "\t" + roi + "_sum\t" + roi + "_pct";
						}
					}
				subject_report_writer.write(header);
				}
		
			boolean init_frequencies = false;
			
			// For each subject, do self- and surface-surface comparison
			for (int i = 0; i < subjects.length; i++){
				
				if (this.is_interrupted){
					if (subject_report_writer != null)
						subject_report_writer.close();
					log("surface_intersections_report_from_files: Process was interrupted.");
					return false;
					}
				
				String subject = subjects[i];
				String level = null;
				if (levels_map != null)
					level = levels_map.get(subject);
				
				File intersection_dir = new File (civet_dir.getAbsolutePath() + File.separator + subject + File.separator + "intersections");
				if (intersection_dir.exists()){
					
					String[] files = intersection_dir.list();
					File intersection_file = null;
					for (int j = 0; intersection_file == null && j < files.length; j++){
						if (files[j].endsWith(intersection_file_pattern))
							intersection_file = new File(intersection_dir.getAbsolutePath() + File.separator + files[j]);
						}
					
					if (intersection_file != null){
						MincSurfaceDataLoader loader = new MincSurfaceDataLoader(intersection_file);
						ArrayList<MguiNumber> vertex_values = loader.loadValues();
						if (vertex_values != null){
							ArrayList<Integer> intersect_vertices = new ArrayList<Integer>();
							for (int j = 0; j < vertex_values.size(); j++){
								if (vertex_values.get(j).getValue() > 0)
									intersect_vertices.add(j);
								}
							
							// Summary stats
							total_intersections += intersect_vertices.size();
							
							// ROI-wise stats
							HashMap<String,MguiInteger> roi_subject = null;
							if (rois != null){
								roi_subject = new HashMap<String,MguiInteger>();
								ArrayList<String> names = new ArrayList<String>(roi_name_map.getNames());
								Collections.sort(names);
								for (int j = 0; j < names.size(); j++){
									String roi = names.get(j);
									MguiInteger stats = new MguiInteger(0);
									MguiInteger total_stats = roi_stats.get(roi);
									roi_subject.put(roi, stats);
									
									for (int k = 0; k < intersect_vertices.size(); k++){
										MguiInteger index = (MguiInteger)rois.get(intersect_vertices.get(k));
										if (index != null){
											String _roi = roi_name_map.get(index.getInt());
											if (_roi != null && _roi.equals(roi)){
												stats.add(1);
												total_stats.add(1);
												}
											}
										}
									}
								}
									
							// Subject-wise stats to file
							if (subject_report_file != null){
								String line = "\n" + subject + "\t";
								if (levels_map != null)
									line = line + levels_map.get(subject); 
								line = line + "\t" + intersect_vertices.size();
									
								if (rois != null){
									ArrayList<String> names = new ArrayList<String> (roi_name_map.getNames());
									Collections.sort(names);
									for (int j = 0; j < names.size(); j++){
										String roi = names.get(j);
										double pct_size = roi_sizes.get(roi).getValue() / 100; 
										MguiInteger stats = roi_subject.get(roi);
										line = line + "\t" + stats.getInt();
										line = line + "\t" + stats.getValue() / pct_size;
										}
									}
								subject_report_writer.write(line);
								}
							
							// Vertex wise frequencies
							if (frequencies != null){
								if (!init_frequencies){
									init_frequencies = true;
									// Init arrays on first iteration
									for (int j = 0; j < vertex_values.size(); j++){
										frequencies.add(new MguiFloat(0));
										if (levels != null){
											for (int k = 0; k < levels.size(); k++){
												String _level = levels.get(k);
												frequencies_levels.get(_level).add(new MguiFloat(0));
												}
											}
										}
									}
								
								for (int j = 0; j < intersect_vertices.size(); j++){
									frequencies.get(intersect_vertices.get(j)).add(1);
									if (level != null)
										frequencies_levels.get(level).get(intersect_vertices.get(j)).add(1);
									}
								}
							
							// Levels stats
							if (levels_map != null && level != null){
								level_sizes.get(level).add(1);
								levels_frequencies.get(level).add(intersect_vertices.size());
								
								// ROIs?
								if (roi_levels_frequencies != null){
									HashMap<String,MguiInteger> r_stats = roi_levels_frequencies.get(level);
									
									ArrayList<String> names = new ArrayList<String> (roi_name_map.getNames());
									Collections.sort(names);
									for (int j = 0; j < names.size(); j++){
										String roi = names.get(j);
										MguiInteger stats = r_stats.get(roi);
										MguiInteger s_stats = roi_subject.get(roi);
										
										stats.add(s_stats.getValue());
										}
									}
								}
							
							log("surface_intersections_report_from_files: Processed subject '" + subject + "'.");
							success_count++;
						}else{
							log("surface_intersections_report_from_files: Intersection frequencies could not be loaded for subject '" + subject + "'.");
							fail_count++;
							}
					}else{
						log("surface_intersections_report_from_files: No intersection file found for subject '" + subject + "'.");
						fail_count++;
						}
				}else{
					
					log("surface_intersections_report_from_files: No intersection folder for subject '" + subject + "'.");
					fail_count++;
					}
				
				}
			
			// Finish up
			
			if (subject_report_file != null){
				subject_report_writer.close();
				log("surface_intersections_report_from_files: Wrote subject table report to '" + subject_report_file.getAbsolutePath() + "'.");
				}
			
			// Write summary report
			BufferedWriter report_writer = new BufferedWriter(new FileWriter(report_file));
			double n = success_count; // subjects.length; 
			
			String line = "Level\tN\tintersections_sum\tintersections_pct";
			report_writer.write(line);
			
			double pct_size = n * (double)frequencies.size() / 100.0;
			
			line = "\nOverall\t" + (int)n + "\t" + total_intersections + "\t" + ((double)total_intersections / pct_size * 100.0);	
			report_writer.write(line);
			
			//Levels?
			if (levels_map != null){
				
				for (int i = 0; i < levels.size(); i++){
					String level = levels.get(i);
					pct_size = level_sizes.get(level).getValue() * (double)frequencies.size() / 100.0;
					MguiInteger stats = levels_frequencies.get(level);
					line = "\n" + level + "\t" + level_sizes.get(level).getInt() + "\t" + stats.getInt() + "\t" + stats.getValue() / pct_size;
					report_writer.write(line);
					}
				}
			
			report_writer.close();
			log("surface_intersections_report_from_files: Wrote summary report to '" + report_file.getAbsolutePath() + "'.");
			
			MincSurfaceDataWriter surface_data_writer;
			
			// Write vertex-wise frequencies
			if (frequency_file_sum != null){
				surface_data_writer = new MincSurfaceDataWriter(frequency_file_sum);
				if (!surface_data_writer.writeValues(frequencies))
					log("surface_intersections_report_from_files: Could not write to file '" + frequency_file_sum.getAbsolutePath() + "'.",
							LoggingType.Errors);
				for (int i = 0; i < frequencies.size(); i++)
					frequencies.get(i).divide(n).multiply(100);
				surface_data_writer = new MincSurfaceDataWriter(frequency_file_pct);
				if (!surface_data_writer.writeValues(frequencies))
					log("surface_intersections_report_from_files: Could not write to file '" + frequency_file_pct.getAbsolutePath() + "'.",
							LoggingType.Errors);
				log("surface_intersections_report_from_files: Wrote vertex-wise self-intersection frequency maps.");
				}
			
			// Level-wise frequencies
			if (frequency_files_levels_sum != null){
				
				for (int i = 0; i < levels.size(); i++){
					String level = levels.get(i);
					File file = frequency_files_levels_sum.get(level);
					surface_data_writer = new MincSurfaceDataWriter(file);
					double size = level_sizes.get(level).getValue();
					ArrayList<MguiNumber> list = frequencies_levels.get(level);
					if (!surface_data_writer.writeValues(list))
						log("surface_intersections_report_from_files: Could not write to file '" + file.getAbsolutePath() + "'.",
								LoggingType.Errors);
					file = frequency_files_levels_pct.get(level);
					surface_data_writer = new MincSurfaceDataWriter(file);
					for (int j = 0; j < list.size(); j++)
						list.get(j).divide(size).multiply(100);
					if (!surface_data_writer.writeValues(list))
						log("surface_intersections_report_from_files: Could not write to file '" + file.getAbsolutePath() + "'.",
								LoggingType.Errors);
					}
				
				}
			
			// Write ROI table
			if (roi_report_file != null){
				BufferedWriter writer = new BufferedWriter(new FileWriter(roi_report_file));
				//Header
				line = "Level\tROI\tintersections_sum\tintersections_pct";
				writer.write(line);
				
				ArrayList<String> roi_names = new ArrayList<String>(roi_name_map.getNames());
				Collections.sort(roi_names);
				String level = "Overall";
				
				for (int i = 0; i < roi_names.size(); i++){
					String roi = roi_names.get(i);
					pct_size = roi_sizes.get(roi).getValue() * n / 100.0;
					MguiInteger stats = roi_stats.get(roi);
					
					line = "\n" + level + "\t" + roi + "\t" + stats.getInt() + "\t" + stats.getValue() / pct_size;
					writer.write(line);
					}
				
				//Levels/ROIs
				if (levels != null){
					for (int i = 0; i < levels.size(); i++){
						level = levels.get(i);
						HashMap<String,MguiInteger> map = roi_levels_frequencies.get(level);
						for (int j = 0; j < roi_names.size(); j++){
							String roi = roi_names.get(j);
							pct_size = roi_sizes.get(roi).getValue() * level_sizes.get(level).getValue() / 100.0;
							MguiInteger stats = map.get(roi);
							
							line = "\n" + level + "\t" + roi + "\t" + stats.getInt() + "\t" + stats.getValue() / pct_size;;
							writer.write(line);
							}
						}
					}
				
				writer.close();
				log("surface_intersections_report_from_files: Wrote ROI report to '" + roi_report_file.getAbsolutePath() + "'.");
				}
			
		}catch(IOException e){
			log("surface_intersections_report_from_files: I/O Exception encountered: " + e.getMessage());
			fail_count++;
			}
		
		return true;
	}
	
	/*********************************************************************
	 * Compiles a list of valid subjects from a civet directory, and filters a subject data file by this list.
	 * Creates two tab-delimited output files, with no headers: (1.) a subject id list; and (2.) a subject data list. 
	 * These formats are ideal for Matlab input. 
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>valid_subject_dir: 		Directory containing subject id's as subdirectories; this acts as the filter list
	 * <li>subject_data_file:		Tab-delimited data file with subject id's as the first column; this file will be filtered
	 * <li>subject_output_file:		Output file containing a list of the filtered subject id's
	 * <li>data_output_file:		Output file containing the data from the filtered file (no subject id column and no header 
	 * 								line).
	 * </ul>
	 * 
	 * @return
	 */
	boolean select_valid_subject_data(){
		
		String _param = parameters.get("valid_subject_dir");
		//File valid_files = new File("/export02/data/adni/civet_validated");
		//File subject_file = new File("/export02/data/adni/metadata/adni_subject_data_mci.txt");
		//File subject_output_file = new File("/export02/data/adni/metadata/adni_valid_subject_mci.txt");
		//File data_output_file = new File("/export02/data/adni/metadata/adni_valid_data_mci.txt");
		
		File valid_subject_dir = new File(_param);
		if (_param == null){
			log("select_valid_subject_data: parameter valid_subject_dir not optional.",
					LoggingType.Errors);
				return false;
			}
		if (!valid_subject_dir.exists()){
			log("select_valid_subject_data: valid_subject_dir '" + valid_subject_dir.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("subject_data_file");
		File subject_data_file = new File(_param);
		if (_param == null){
			log("select_valid_subject_data: parameter subject_data_file not optional.",
					LoggingType.Errors);
				return false;
			}
		if (!subject_data_file.exists()){
			log("select_valid_subject_data: subject_data_file '" + subject_data_file.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("subject_output_file");
		File subject_output_file = new File(_param);
		if (_param == null){
			log("select_valid_subject_data: parameter subject_output_file not optional.",
					LoggingType.Errors);
				return false;
			}
		if (subject_output_file.exists() && !subject_output_file.delete()){
			log("select_valid_subject_data: Could not create '" + subject_output_file.getAbsolutePath() + "'.",
				LoggingType.Errors);
			return false;
			}
		
		_param = parameters.get("subject_output_file");
		File data_output_file = new File(_param);
		if (_param == null){
			log("select_valid_subject_data: parameter data_output_file not optional.",
					LoggingType.Errors);
				return false;
			}
		if (data_output_file.exists() && !data_output_file.delete()){
			log("select_valid_subject_data: Could not create '" + data_output_file.getAbsolutePath() + "'.",
				LoggingType.Errors);
			return false;
			}
		
		try{
			String[] dirs = valid_subject_dir.list(IoFunctions.getDirFilter());
			TreeSet<String> subjects = new TreeSet<String>();
			for (int i = 0; i < dirs.length; i++){
				subjects.add(dirs[i]);
				}
			
			BufferedReader reader = new BufferedReader(new FileReader(subject_data_file));
			BufferedWriter subject_writer = new BufferedWriter(new FileWriter(subject_output_file));
			BufferedWriter data_writer = new BufferedWriter(new FileWriter(data_output_file));
			
			int count = 0;
			String line = reader.readLine();
			//writer.write(line);
			line = reader.readLine();
			boolean first = true;
			
			while (line != null){
				int a = line.indexOf("\t");
				String subject = line.substring(0, a);
				if (subjects.contains(subject)){
					if (!first){
						subject_writer.write("\n");
						data_writer.write("\n");
						}
					first = false;
					data_writer.write(line.substring(a + 1));
					subject_writer.write(subject);
					count++;
					}
				line = reader.readLine();
				}
			
			reader.close();
			data_writer.close();
			subject_writer.close();
			
			log("Done: " + count + " lines.");
			
		}catch (IOException ex){
			log("select_valid_subject_data: I/O Exception: " + ex.getMessage());
			return false;
			}
		
		return true;
		
	}
	
	/*********************************************************************
	 * Computes an average minc image from a list of subjects, for a Civet directory
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>subject_dir: 			Civet directory 
	 * <li>image_dir: 				Subdirectory containing image 
	 * <li>image_suffix:			Suffix to match
	 * <li>image_prefix:			Prefix to match
	 * <li>subject_ids:				Text file with subject id's
	 * <li>output_file:				Output file for final averaged image
	 * <li>minc_path:				[Optional] path to minc library
	 * </ul>
	 * 
	 * @return
	 */
	boolean civet_average_image(){
		
		String _param = parameters.get("subject_dir");
		
		File subject_dir = new File(_param);
		if (_param == null){
			log("civet_average_image: parameter subject_dir not optional.",
					LoggingType.Errors);
				return false;
			}
		if (!subject_dir.exists()){
			log("civet_average_image: subject_dir '" + subject_dir.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			} 
		
		String image_dir = parameters.get("image_dir");
		if (image_dir == null){
			log("civet_average_image: parameter image_dir not optional.",
					LoggingType.Errors);
				return false;
			}
		
		String image_suffix = parameters.get("image_suffix");
		if (image_suffix == null){
			log("civet_average_image: parameter image_suffix not optional.",
					LoggingType.Errors);
				return false;
			}
		
		String image_prefix = parameters.get("image_prefix");
		if (image_prefix == null){
			log("civet_average_image: parameter image_prefix not optional.",
					LoggingType.Errors);
				return false;
			}
		
		_param = parameters.get("subject_ids");
		
		File subject_ids = new File(_param);
		if (_param == null){
			log("civet_average_image: parameter subject_ids not optional.",
					LoggingType.Errors);
				return false;
			}
		if (!subject_dir.exists()){
			log("civet_average_image: subject_ids '" + subject_ids.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			} 
		
		_param = parameters.get("output_file");
		
		File output_file = new File(_param);
		if (_param == null){
			log("civet_average_image: parameter output_file not optional.",
					LoggingType.Errors);
				return false;
			}
		if (output_file.exists() && !output_file.delete()){
			log("civet_average_image: Can't overwrite '" + output_file.getAbsolutePath() + "'.",
				LoggingType.Errors);
			return false;
			} 
		
		String minc_library = parameters.get("minc_library");
		if (minc_library == null)
			minc_library = "";
		else if (!minc_library.endsWith(File.separator))
			minc_library=minc_library + File.separator;
		
		ArrayList<String> subjects = null;
		
		try{
			BufferedReader reader = new BufferedReader(new FileReader(subject_ids));
			
			subjects = new ArrayList<String>();
			String line = reader.readLine();
			
			while (line != null){
				subjects.add(line);
				line = reader.readLine();
				}
			
			reader.close();
			
		}catch (IOException ex){
			log("civet_average_image: Error reading subject ids.",
					LoggingType.Errors);
				return false;
			}
		
		try{
			
			String temp_image_1 = subject_dir.getAbsolutePath() + File.separator + "temp.mnc";
			String temp_image_2 = subject_dir.getAbsolutePath() + File.separator + "temp2.mnc";
			int success_count = 0, fail_count = 0;
			
			for (int i = 0; i < subjects.size(); i++){
				
				if (this.is_interrupted){
					log("civet_average_image: Process was interrupted.");
					return false;
					}
				
				String subject = subjects.get(i);
				String image_path = subject_dir.getAbsolutePath() + File.separator + 
									subject + File.separator +
									image_dir + File.separator +
									image_prefix + subject + image_suffix;
				
				if (i > 0){
					
					// Perform average operation here
					String command = minc_library + "mincmath -clobber -add " + temp_image_1 + " " + image_path + " " + temp_image_2;
					
					ProcessState state = ProcessState.NotStarted;
					
					try{
						state = CommandFunctions.callAndWait(command);
//						CommandFunctions.call(command);
					}catch (IOException ex){
						state = ProcessState.TerminatedWithError;
						}
					
					if (state == ProcessState.TerminatedNormally){
						log("civet_average_image: Added subject '" + subject + "'.");
						success_count++;
					}else{
						log("civet_average_image: Failed to add subject '" + subject + "'.",
								LoggingType.Errors);
						fail_count++;
						}
					
					IoFunctions.copyFile(new File(temp_image_2), new File(temp_image_1));
					
				}else{
					// First image is initial temp image
					IoFunctions.copyFile(new File(image_path), new File(temp_image_1));
					}
				
				}
			
			// Finally, divide
			String command = minc_library + "mincmath -clobber -div -const " + success_count + " " + temp_image_1 + " " + output_file;
			
			ProcessState state = ProcessState.NotStarted;
			
			try{
				state = CommandFunctions.callAndWait(command);
//				CommandFunctions.call(command);
			}catch (IOException ex){
				state = ProcessState.TerminatedWithError;
				}
			
			if (state == ProcessState.TerminatedNormally){
				log("civet_average_image: Done.");
			}else{
				log("civet_average_image: Failed to divide sum image.",
						LoggingType.Errors);
				}
			
			// Delete temp file
			if (!(new File(temp_image_1)).delete() || !(new File(temp_image_2)).delete()){
				log("civet_average_image: Failed to delete temp image.",
						LoggingType.Errors);
				}
				
			
		}catch (Exception ex){
			log("civet_average_image: Error encountered: " + ex.getMessage(),
					LoggingType.Errors);
				return false;
			}
		
		return true;
	}
	
	/******************************************
	 * Projects a specified surface-based atlas to individual subject voxel space, 
	 * using a Gaussian nearest-neighbour approach. Resulting images will be in Nifti format,
	 * integer data type.
	 * 
	 * <p>For filename patterns: 
	 * <p>{root} is replaced by the subject dir
	 * <br>{subject} is replaced by the subject id
	 * 
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>subject_dir: 			Civet directory; patterned as {root}
	 * <li>surface_dir: 			[Optional] Surface subdirectory; default is "surfaces"
	 * <li>surface_file_pattern:  	The surface filename pattern to match; if {@code merge_surfaces} is true, 
	 * 								expects to find a left and right surface here
	 * <li>merge_surfaces: 			[Flag] If set, will merge surfaces in the order left, right 
	 * <li>atlas_file:				Path to the vertex-wise atlas parcellation
	 * <li>template_image: 			Image from which to obtain the volume dimensions, etc. (nifti)
	 * <li>output_dir: 				[Optional] Output subdirectory; default is "atlases"
	 * <li>output_pattern:			File pattern with which to write the resulting image (nifti)
	 * <li>param_file:	 			[Optional] Set parameters for the mesh -> volume function
	 * 								(See {@link MeshFunctions.mapMeshToGrid3DGaussian});
	 * 								default parameters are: sigma_normal=1.5, sigma_tangent=2.0,
	 * 								sigma_max_normal=1.5, sigma_max_tangent=2.0, normalize=false,
	 * 								is_discrete=false
	 * <li>skip_existing: 			Indicates that existing target files should be skipped
	 * </ul>
	 * 
	 * 
	 * @return
	 */
	boolean civet_surface_atlas_to_volume(){
		
		String _param = parameters.get("subject_dir");
		
		if (_param == null){
			log("civet_surface_atlas_to_volume: parameter subject_dir not optional.",
					LoggingType.Errors);
				return false;
			}
		File subject_dir = new File(_param);
		if (!subject_dir.exists()){
			log("civet_surface_atlas_to_volume: subject_dir '" + subject_dir.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			} 
		
		_param = parameters.get("atlas_file");
		
		if (_param == null){
			log("civet_surface_atlas_to_volume: parameter atlas_file not optional.",
					LoggingType.Errors);
				return false;
			}
		File atlas_file = new File(_param);
		if (!atlas_file.exists()){
			log("civet_surface_atlas_to_volume: atlas_file '" + atlas_file.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			} 
		
		_param = parameters.get("template_image");
		
		if (_param == null){
			log("civet_surface_atlas_to_volume: parameter atlas_file not optional.",
					LoggingType.Errors);
				return false;
			}
		File template_image = new File(_param);
		if (!template_image.exists()){
			log("civet_surface_atlas_to_volume: template_image '" + template_image.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			} 
		
		// Projection parameters; defaults:
		String[] temp = new String[]{"-sigma_normal", "1.5",
					  "-sigma_tangent", "2.0", 
					  "-sigma_max_normal", "1.5", 
					  "-sigma_max_tangent", "2.0",
					  "-is_discrete"};
		
		_param = parameters.get("param_file");
		
		// Load from file?
		if (_param != null){
			File param_file = new File(_param);
			if (!param_file.exists()){
				log("civet_surface_atlas_to_volume: Cannot find parameter file: " + param_file.getAbsolutePath(),
						LoggingType.Errors);
				return false;
				}
			try{
				String[] params2 = getParametersFromFile(param_file);
				String[] temp2 = new String[temp.length + params2.length];
				System.arraycopy(temp, 0, temp2, 0, temp.length);
				System.arraycopy(params2, 0, temp2, temp.length, params2.length);
				temp = temp2;
			}catch (IOException ex){
				ex.printStackTrace();
				return false;
				}
			}
		
		boolean skip_existing = parameters.containsKey("skip_existing");
		
		HashMap<String, String> proj_params = getParameters(temp);
		
		String surface_dir = parameters.get("surface_dir");
		if (surface_dir == null){
			surface_dir = "surfaces";
			}
		
		String surface_file_pattern = parameters.get("surface_file_pattern");
		
		if (surface_file_pattern == null){
			log("civet_surface_atlas_to_volume: parameter surface_file_pattern not optional.",
					LoggingType.Errors);
				return false;
			}
		String s = surface_file_pattern.replace("{root}", subject_dir.getAbsolutePath());
		while (!s.equals(surface_file_pattern)){
			surface_file_pattern = s;
			s = surface_file_pattern.replace("{root}", subject_dir.getAbsolutePath());
			}
		
		String output_dir = parameters.get("output_dir");
		if (output_dir == null){
			output_dir = "atlases";
			}
		
		String output_pattern = parameters.get("output_pattern");
		
		if (output_pattern == null){
			log("civet_surface_atlas_to_volume: parameter output_pattern not optional.",
					LoggingType.Errors);
				return false;
			}
		
		s = output_pattern.replace("{root}", subject_dir.getAbsolutePath());
		while (!s.equals(output_pattern)){
			output_pattern = s;
			s = output_pattern.replace("{root}", subject_dir.getAbsolutePath());
			}
		
		boolean merge_surfaces = parameters.containsKey("merge_surfaces");
		
		// First off:
		// 1. Load atlas
		MincSurfaceDataLoader sd_loader = new MincSurfaceDataLoader(atlas_file); 
		ArrayList<MguiNumber> atlas_values = sd_loader.loadValues(null);
		
		// 2. Get template image parameters
		NiftiVolumeLoader volume_loader = new NiftiVolumeLoader(template_image);
		VolumeMetadata template_metadata = null;
		
		try{
			template_metadata = volume_loader.getVolumeMetadata();
		}catch (Exception e){
			log("civet_surface_atlas_to_volume: Could not read the template image '" + template_image.getAbsolutePath() + 
				"'.\nDetails: " + e.getMessage(),
				LoggingType.Errors);
			return false;
			}
		
		VolumeInputOptions template_options = new VolumeInputOptions();
		
		// TODO: add these as parameter flags?
		//options.flip_x = chkFlipX.isSelected();
		//options.flip_y = chkFlipY.isSelected();
		//options.flip_z = chkFlipZ.isSelected();
		int[] dims = template_metadata.getDataDims();
		float[] geom = template_metadata.getGeomDims();
		Point3f origin = template_metadata.getOrigin();
	
		template_options.set_dims = true;
		template_options.dim_x = dims[0];
		template_options.dim_y = dims[1];
		template_options.dim_z = dims[2];
		template_options.set_geom = true;
		template_options.geom_x = geom[0];
		template_options.geom_y = geom[1];
		template_options.geom_z = geom[2];
		template_options.transferType = template_metadata.getDataType();
		template_options.set_origin = true;
		template_options.origin_x = origin.x;
		template_options.origin_y = origin.y;
		template_options.origin_z = origin.z;
		template_options.axis_x = new Vector3f(1, 0, 0);
		template_options.axis_y = new Vector3f(0, 1, 0);
		template_options.axis_z = new Vector3f(0, 0, 1);
		template_options.set_type = true;
		
		// 3. Get surface loader options
		SurfaceInputOptions surface_options = new SurfaceInputOptions();
		surface_options.merge_shapes = merge_surfaces;
		
		// 4. Set up volume output options
		VolumeOutputOptions volume_output_options = new VolumeOutputOptions();
		volume_output_options.datatype = Nifti1Dataset.NIFTI_TYPE_INT32;
		volume_output_options.channel = "default";
		volume_output_options.apply_masks = false;
		volume_output_options.flipY = true;
		
		// Loaders
		MincSurfaceLoader minc_loader = new MincSurfaceLoader();
//		volume_loader = null;
		NiftiVolumeWriter volume_writer = new NiftiVolumeWriter();
		
		// Shapes
		Mesh3DInt mesh_int = null;
		Grid3D grid3d = null;
		Volume3DInt volume_int = null;
		
		// Let's go
		String[] subjects = subject_dir.list(getDirFilter());
		int success_count = 0, fail_count = 0, skip_count = 0;
		boolean success = true;
		
		for (int i = 0; i <  subjects.length; i++){
			
			if (this.is_interrupted){
				log("civet_surface_atlas_to_volume: Process was interrupted.", LoggingType.Errors);
				return false;
				}
			
			// For each subject:
			boolean failed = false, skip = false;;
			String subject = subjects[i];
			log("civet_surface_atlas_to_volume: Processing subject '" + subject + "'.",
					LoggingType.Errors);
			
			// 1. Load (merged) surface(s)
			s = surface_file_pattern.replace("{subject}", subject);
			String s2 = surface_file_pattern;
			while (!s.equals(s2)){
				s2 = s;
				s = s2.replace("{subject}", subject);
				}
			final String pattern = s;
			File surf_dir = new File(subject_dir.getAbsolutePath() + File.separator + 
									 subject + File.separator + 
									 surface_dir);
			
			if (!surf_dir.exists()){
				log("civet_surface_atlas_to_volume: Surface folder '" + surf_dir.getAbsolutePath() + "' not found!",
						LoggingType.Errors);
				failed = true;
				}
			
			File output_file = null;
			
			if (!failed){
				s = output_pattern.replace("{subject}", subject);
				s2 = output_pattern;
				while (!s.equals(s2)){
					s2 = s;
					s = s2.replace("{subject}", subject);
					}
				File _output_dir = new File(subject_dir.getAbsolutePath() + File.separator + subject + File.separator + output_dir);
				if (!_output_dir.exists() && !_output_dir.mkdir()){
					log("civet_surface_atlas_to_volume: Could not create folder '" + _output_dir.getAbsolutePath() + "'.",
							LoggingType.Errors);
					failed = true;
				}else{
					output_file = new File(_output_dir.getAbsolutePath() + File.separator + s);
					if (output_file.exists()){
						if (skip_existing){
							skip = true;
						}else if (!output_file.delete()){
							log("civet_surface_atlas_to_volume: Could not delete existing file '" + output_file.getAbsolutePath() + "'.",
									LoggingType.Errors);
							failed = true;
							}
						}
					}
				}
				
			if (!failed && !skip){
				String[] files = surf_dir.list(new FilenameFilter(){
					public boolean accept(File dir, String name) {
			            return name.contains(pattern) && name.endsWith(".obj");
			        }
				});
					
				if (files.length == 0){
					log("civet_surface_atlas_to_volume: No surfaces found for subject '" + subject + "'.",
							LoggingType.Errors);
					failed = true;
				}else{
					// File loader options
					File[] ffiles = new File[files.length];
					for (int f = 0; f < files.length; f++)
						ffiles[f] = new File(surf_dir.getAbsolutePath() + File.separator + files[f]);
					surface_options.setFiles(ffiles);
					surface_options.shapeSet = new ShapeSet3DInt();
					if (!minc_loader.load(surface_options)){
						log("civet_surface_atlas_to_volume: Could not load surfaces for '" + subject + "'.",
								LoggingType.Errors);
						failed = true;
						}
					}
				}
			
			if (!(failed || skip) && (surface_options.shapeSet == null || surface_options.shapeSet.getSize() != 1)){
				log("civet_surface_atlas_to_volume: Invalid number of surfaces for subject '" + subject + "' (" + surface_options.shapeSet.getSize() +
					"). Did you try using -merge_surfaces?",
					LoggingType.Errors);
				failed = true;
				}
			
			// 2. Add atlas column
			if (!failed && !skip){
				mesh_int = (Mesh3DInt)surface_options.shapeSet.getMember(0);
				if (!mesh_int.addVertexData("atlas", atlas_values)){
					log("civet_surface_atlas_to_volume: Could not set atlas values for '" + subject + "'. (n=" + mesh_int.getMesh().getSize() + ", m=" +
							atlas_values.size() + ")",
							LoggingType.Errors);
					failed = true;
					}
				}
			
			// 3. Create new volume
			if (!failed && !skip){
				grid3d = getGrid(template_options);
				}
			
			// 4. Project atlas to volume
			if (!failed && !skip){
				try{
				grid3d = MeshFunctions.mapMeshToGrid3DGaussian(mesh_int,
												  			   grid3d,
												  			   "atlas",
												  			   "default",
												  			   Float.valueOf(proj_params.get("sigma_normal")),
												  			   Float.valueOf(proj_params.get("sigma_tangent")),
												  			   Float.valueOf(proj_params.get("sigma_max_normal")),
												  			   Float.valueOf(proj_params.get("sigma_max_tangent")),
												  			   proj_params.containsKey("normalize"),
												  			   proj_params.containsKey("is_discrete"),
												  			   null);
				}catch (Exception ex){
					log("civet_surface_atlas_to_volume: Projection failed for '" + subject + "'.\nDetails: " + ex.getMessage(),
							LoggingType.Errors);
					failed = true;
					}
				
				if (!failed && grid3d == null){
					log("Projection failed for '" + subject + "'.",
							LoggingType.Errors);
					failed = true;
					}
				
				}
			
			// 5. Write volume
			if (!failed && !skip){
				volume_writer.setFile(output_file);
				volume_output_options.volume = new Volume3DInt(grid3d, false);
				try{
					volume_writer.writeVolume(volume_output_options);
				}catch (IOException ex){
					log("civet_surface_atlas_to_volume: Could not write to '" + output_file.getAbsolutePath() + "'.\nDetails: " +
						ex.getMessage(),
						LoggingType.Errors);
					failed = true;
					}
				}
		
			// 6. Clean up
			if (mesh_int != null){
				mesh_int.destroy();
				mesh_int = null;
				}
		
			if (volume_output_options.volume != null){
				volume_output_options.volume.destroy();
				volume_output_options.volume = null;
				}
			
			if (failed){
				log("civet_surface_atlas_to_volume: Subject '" + subject + "': failed.", LoggingType.Errors);
				fail_count++;
				success = false;
			}else if (skip){
				log("civet_surface_atlas_to_volume: Subject '" + subject + "' exists: skipped.");
				skip_count++;
			}else{
				log("civet_surface_atlas_to_volume: Subject '" + subject + "': success.");
				success_count++;
				}
			
			}
		
		log("civet_surface_atlas_to_volume: Done. " + success_count + " successes. " + fail_count + " failures. " + skip_count + " skipped.");
		
		return success;
	}
	
	private Grid3D getGrid(VolumeInputOptions options){
		
		Box3D box = new Box3D();
		box.setBasePt(new Point3f(options.origin_x,
								  options.origin_y,
								  options.origin_z));

		Vector3f v = options.axis_x;
		v.normalize();
		v.scale(options.geom_x);
		box.setSAxis(v);
		
		v = options.axis_y;
		v.normalize();
		v.scale(options.geom_y);
		box.setTAxis(v);
		
		v = options.axis_z;
		v.normalize();
		v.scale(options.geom_z);
		box.setRAxis(v);
				
		Grid3D grid = new Grid3D(options.dim_x,
					 		     options.dim_y,
					 		     options.dim_z,
					 		     1,
					 		     box);
		
		return grid;
	}
	
	/******************************************
	 * Generates volume masks -- one boolean volume per region -- for a given atlas and parcellation.
	 *  
	 * <p>Parameters:
	 * 
	 * <ul>
	 * <li>subject_dir: 			Subject directory containing subject-wise atlas volumes (nifti)
	 * <li>name_map_file: 			Path to the name map used to define the masks
	 * <li>atlas_file_pattern: 		Pattern for atlas volume file; use {subject} to insert subject ID
	 * <li>output_dir: 				Output directory (subject directories)
	 * <li>output_pattern: 			Pattern for the output files; use {subject} to insert subject ID, {roi} to insert
	 * 								ROI name
	 * <li>gzip: 					[Flag] Whether to compress the resulting nifti files with gzip
	 * </ul>
	 * 
	 * 
	 * @return
	 */
	boolean create_volume_atlas_masks(){
		
		String _param = parameters.get("subject_dir");
		
		if (_param == null){
			log("create_volume_atlas_masks: parameter subject_dir not optional.",
					LoggingType.Errors);
				return false;
			}
		File subject_dir = new File(_param);
		if (!subject_dir.exists()){
			log("create_volume_atlas_masks: subject_dir '" + subject_dir.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			} 
		
		_param = parameters.get("name_map_file");
		
		if (_param == null){
			log("create_volume_atlas_masks: parameter name_map_file not optional.",
					LoggingType.Errors);
				return false;
			}
		File name_map_file = new File(_param);
		if (!name_map_file.exists()){
			log("create_volume_atlas_masks: name_map_file '" + name_map_file.getAbsolutePath() + "' doesn't exist.",
				LoggingType.Errors);
			return false;
			}
		
		NameMapLoader nm_loader = new NameMapLoader(name_map_file);
		NameMap name_map = null;
		
		try{
			name_map = (NameMap)nm_loader.loadObject();
		}catch (IOException ex){
			log("create_volume_atlas_masks: Erroring reading name map file '" + name_map_file.getAbsolutePath() + "'.\nDetails: " +
				ex.getMessage(),
				LoggingType.Errors);
				return false;
			}
		
		ArrayList<Integer> atlas_indices = name_map.getIndices();
		
		_param = parameters.get("output_dir");
		if (_param == null){
			log("create_volume_atlas_masks: parameter output_dir not optional.",
				LoggingType.Errors);
				return false;
			}
		
		File output_dir = new File(_param);
		if (!output_dir.exists() && !output_dir.mkdir()){
			log("create_volume_atlas_masks: Could not create output folder '" + output_dir.getAbsolutePath() + "'.",
				LoggingType.Errors);
				return false;
			}
		
		String atlas_file_pattern = parameters.get("atlas_file_pattern");
		if (atlas_file_pattern == null){
			log("create_volume_atlas_masks: parameter atlas_file_pattern not optional.",
					LoggingType.Errors);
					return false;
			}
		
		String output_pattern = parameters.get("output_pattern");
		if (output_pattern == null){
			log("create_volume_atlas_masks: parameter output_pattern not optional.",
					LoggingType.Errors);
					return false;
			}
		
		boolean gzip = parameters.containsKey("gzip");
		
		String[] subjects = subject_dir.list(IoFunctions.getDirFilter());
		
		// Loaders/writers
		NiftiVolumeWriter volume_writer = new NiftiVolumeWriter();
		NiftiVolumeLoader volume_loader = new NiftiVolumeLoader();
		
		// Shapes
		Grid3D atlas_grid = null;
		
		int success_count = 0, fail_count = 0;
		
		for (int i = 0; i <  subjects.length; i++){
			
			boolean failed = false;
			
			if (this.is_interrupted){
				log("create_volume_atlas_masks: Process was interrupted.",
						LoggingType.Errors);
				return false;
				}
			
			String subject = subjects[i];
			
			// 1. Load atlas volume
			String s = atlas_file_pattern.replace("{subject}", subject);
			String s2 = atlas_file_pattern;
			while (!s.equals(s2)){
				s2 = s;
				s = s2.replace("{subject}", subject);
				}
			File atlas_file = new File(subject_dir.getAbsolutePath() + File.separator + subject + File.separator + s);
			NiftiMetadata grid_metadata = null;
			
			if (!atlas_file.exists()){
				log("create_volume_atlas_masks: Atlas file '" + atlas_file.getAbsolutePath() + "' not found.",
						LoggingType.Errors);
				failed = true;
			}else{
				volume_loader.setFile(atlas_file);
				try{
					atlas_grid = volume_loader.getGrid3D(0);
					grid_metadata = (NiftiMetadata)volume_loader.getVolumeMetadata();
					grid_metadata.setDataType(Nifti1Dataset.NIFTI_TYPE_INT32);
				}catch (Exception ex){
					log("create_volume_atlas_masks: Could not load volume '" + atlas_file.getAbsolutePath() + "'.\nDetails: " +
						ex.getMessage(),
						LoggingType.Errors);
					failed = true;
					}
				}
			
			// 1.b. Create output subject dir
			File os_dir = new File(output_dir, subject);
			if (!os_dir.exists() && !os_dir.mkdir()){
				log("create_volume_atlas_masks: Could not create output folder '" + os_dir.getAbsolutePath() + "'.",
						LoggingType.Errors);
				failed = true;
			}
			
			// 2. For each ROI
			if (!failed){
				s = output_pattern.replace("{subject}", subject);
				s2 = output_pattern;
				while (!s.equals(s2)){
					s2 = s;
					s = s2.replace("{subject}", subject);
					}
				String subject_output_pattern = s;
				
				// 2.1. Mask by ROI
				for (int r = 0; r < atlas_indices.size(); r++){
					
					if (this.is_interrupted){
						log("create_volume_atlas_masks: Process was interrupted.",
								LoggingType.Errors);
						return false;
						}
					
					int roi = atlas_indices.get(r);
					Grid3D mask_grid = VolumeFunctions.getIntMaskedByValue(atlas_grid, roi, true);
					
					// 2.2. Write resulting volume (gzip if necessary)
					String roi_name = name_map.get(roi);
					s = subject_output_pattern.replace("{roi}", roi_name);
					s2 = subject_output_pattern;
					while (!s.equals(s2)){
						s2 = s;
						s = s2.replace("{roi}", roi_name);
						}
					
					File output_file = new File(os_dir.getAbsolutePath() + File.separator + s);
					if (output_file.exists() && !output_file.delete()){
						log("create_volume_atlas_masks: Could not create file '" + output_file.getAbsolutePath() + "'.",
							LoggingType.Errors);
						failed = true;
					}else{
						try{
							volume_writer.setFile(output_file);
							volume_writer.writeGrid(mask_grid, grid_metadata);
							log("create_volume_atlas_masks: Wrote '" + output_file.getAbsolutePath() + "'.");
							if (gzip){
								File gzip_file = new File(output_file.getAbsolutePath() + ".gz");
								IoFunctions.gzipFile(output_file, gzip_file);
								output_file.delete();
								}
						}catch (IOException ex){
							log("create_volume_atlas_masks: Erroring writing volume to '" + output_file.getAbsolutePath() + "'.\nDetails: " +
								ex.getMessage(),
								LoggingType.Errors);
							failed = true;
							}
						}
									
					}
				}
			
			if (failed){
				log("create_volume_atlas_masks: Subject '" + subject + "': failed.",
					LoggingType.Errors);
				fail_count++;
			}else{
				log("create_volume_atlas_masks: Subject '" + subject + "': success.");
				success_count++;
				}
			
			// 3. Clean up
			
			
			}
		
		log("create_volume_atlas_masks: Done. " + success_count + " successes. " + fail_count + " failures.");
		
		return true;
	}
	
	class SurfaceIntersectionStats{
	
		public int self_vertices_1 = 0;
		public int self_faces_1 = 0;
		public int self_vertices_2 = 0;
		public int self_faces_2 = 0;
		public int surf_vertices_1 = 0;
		public int surf_faces_1 = 0;
		public int surf_vertices_2 = 0;
		public int surf_faces_2 = 0;
		
	}
	
	class AdniStats{
		
		public int subjects_in_list = 0;
		public HashMap<String,HashMap<String,Integer>> frequency_map = new HashMap<String,HashMap<String,Integer>>();
		public TreeSet<String> subjects = new TreeSet<String>();
		
	}
	
	public static FilenameFilter getDirFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return (new File(dir + File.separator + name)).isDirectory();
	        }
		};
	}
	
	public static FilenameFilter getFailedFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".failed");
	        }
		};
	}
	
	public static FilenameFilter getObjFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".obj");
	        }
		};
	}
	
	public static FilenameFilter getNiftiFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".nii") || name.endsWith(".hdr");
	        }
		};
	}
	
	public static FilenameFilter getMincFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".mnc");
	        }
		};
	}
	
	public static FilenameFilter getMincFilter(final String match){
		if (match == null) return getMincFilter();
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".mnc") && name.contains(match);
	        }
		};
	}
	
	public static FilenameFilter getExtFilter(final String extension){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	        	return name.endsWith("." + extension);
	        }
		};
	}
	
	public static FilenameFilter getExtFilter(final String extension, final String match){
		if (match == null) return getExtFilter(extension);
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith("." + extension) && name.contains(match);
	        }
		};
	}
	
	public static FilenameFilter getXmlFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".xml");
	        }
		};
	}
	
	public static FilenameFilter getXfmFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".xfm");
	        }
		};
	}
	
}
